package hu.oroscom.c1801.changelog;

import hu.oroscom.c1801.domain.BaseEntity;
import hu.oroscom.c1801.domain.EntityBase;
import hu.oroscom.c1801.domain.User;
import hu.oroscom.c1801.util.UserUtil;

import java.util.*;

/**
 * This class is typically been called in a JSF backing bean before reading
 * a database entity which is being edited. This information is stored
 * in a ThreadLocal and will be accessed from inside the entities to
 * create historisation data.
 *
 * @author <a href="mailto:struberg@yahoo.de">Mark Struberg</a>
 */
public final class Editor {
	private static ThreadLocal<EditInfo> editInfoThreadLocal = new ThreadLocal<>();

	/**
	 * By calling this method, the given entities will be under
	 * suspicion of being edited if (and only if) the user has
	 * proper permission to change the entity
	 * <p/>
	 * This method should only be called if the current user has
	 * sufficient rights to change the given entities at all.
	 *
	 * @param user           the User who performs the editing
	 * @param editedEntities the entities whose changes should get tracked
	 */
	public static <T extends EntityBase> void markEditable(User user, Class<T>... editedEntities) {
		EditInfo editInfo = editInfoThreadLocal.get();

		if (editInfo == null) {
			editInfo = new EditInfo();

			if (user != null) {
				editInfo.user = user;
			} else {
				editInfo.user = UserUtil.getUser();
			}
			editInfo.edited = new HashSet<>();
			editInfo.unmarked = new HashSet<>();
			editInfo.oldData = new HashMap<>();

			editInfoThreadLocal.set(editInfo);
		}

		for (Class<T> historisedEntity : editedEntities) {
			editInfo.edited.add(historisedEntity);
		}
	}

	public static <T extends EntityBase> Set<Class<T>> getEdited() {
		EditInfo editInfo = editInfoThreadLocal.get();

		if (editInfo != null) {
			return editInfo.edited;
		}

		return null;
	}


	/**
	 * Call this method to disable the collection of the oldValues for the current thread.
	 * Any previously edited entities will still contain the oldValues and will therefore
	 * get tracked.
	 *
	 * @param uneditedEntities
	 */
	public static <T extends EntityBase> void unmarkEditable(Class<T>... uneditedEntities) {
		EditInfo editInfo = editInfoThreadLocal.get();

		if (editInfo != null) {
			for (Class<T> unmarkedEntity : uneditedEntities) {
				editInfo.unmarked.add(unmarkedEntity);
			}
		}
	}

	/**
	 * Determine whether an entity is unmarked again.
	 * very useful after storing and before forwarding.
	 */
	public static <T extends EntityBase> boolean isUnmarked(T he) {
		EditInfo editInfo = editInfoThreadLocal.get();
		if (editInfo == null) {
			return false;
		}

		if (editInfo.unmarked != null && !editInfo.unmarked.isEmpty()) {
			return editInfo.unmarked.contains(he.getClass());
		}

		return false;
	}


	/**
	 * This has to be called at the end of each request to ensure
	 * that the ThreadLocal editor information get's cleared properly.
	 */
	public static void cleanupThread() {
		if (editInfoThreadLocal.get() != null) {
			editInfoThreadLocal.set(null);
			editInfoThreadLocal.remove();
		}
	}

	/**
	 * @param he the entity which should get checked if it's currently being edited
	 * @return <code>true</code> if the given entity is currently being edited
	 * and thus should get editor.
	 */
	public static <T extends EntityBase> boolean isEdited(T he) {
		EditInfo editInfo = editInfoThreadLocal.get();
		if (editInfo == null) {
			return false;
		}

		if (editInfo.edited != null && !editInfo.edited.isEmpty()) {
			return editInfo.edited.contains(he.getClass());
		}

		return false;
	}

	public static <T extends EntityBase> void setOldData(T he, Map<String, Object> oldData) {
		EditInfo editInfo = editInfoThreadLocal.get();
		if (editInfo == null) {
			throw new RuntimeException("Entity not in edit mode! " + he.getClass().getName() + " id=" + he.hashCode());
		}

		Object key = getEntityKey(he);

		editInfo.oldData.put(key, oldData);
	}

	/**
	 * @param he
	 * @return the entity key which consists of the ClassName + the id
	 */
	private static <T extends EntityBase> Object getEntityKey(T he) {
		if (he instanceof BaseEntity) {
			BaseEntity be = (BaseEntity) he;
			if (be.getId() == null) {
				return he;
			} else {
				return he.getClass().getName() + "/" + he.hashCode();
			}
		} else {
			return he.getClass().getName() + "/" + he.hashCode();
		}
	}

	public static <T extends EntityBase> Map<String, Object> getOldData(T he) {
		EditInfo editInfo = editInfoThreadLocal.get();
		if (editInfo == null) {
			throw new RuntimeException("Entity not in edit mode! " + he.getClass().getName() + " id=" + he.hashCode());
		}

		Object key = getEntityKey(he);

		Map<String, Object> oldData = (Map<String, Object>) editInfo.oldData.get(key);

		if (oldData == null) {
			// fallback to object
			oldData = (Map<String, Object>) editInfo.oldData.get(he);
		}

		return oldData;
	}

	public static User getUser() {
		EditInfo editInfo = editInfoThreadLocal.get();
		if (editInfo == null) {
			return null;
		}
		return editInfo.user;
	}

	/**
	 * Maintain a constant transaction timestamp which will be
	 * used for all changes in a transaction.
	 */
	public static Date getTransactionTst() {
		EditInfo editInfo = editInfoThreadLocal.get();
		if (editInfo == null) {
			return null;
		}

		Date tst = editInfo.transactionTst;
		if (tst == null) {
			tst = new Date();
			editInfo.transactionTst = tst;
		}

		return tst;
	}


	/**
	 * internal data container.
	 */
	private static class EditInfo<T extends EntityBase> {
		public User user;
		public Locale locale;
		public Set<Class<T>> edited;
		public Set<Class<T>> unmarked;
		public Map<Object, Map<String, Object>> oldData;
		public Date transactionTst;
	}

}
