package hu.oroscom.c1801.changelog;

import hu.oroscom.c1801.Application;
import hu.oroscom.c1801.domain.BaseEntity;
import hu.oroscom.c1801.domain.EntityBase;
import hu.oroscom.c1801.domain.User;
import hu.oroscom.c1801.enumeration.EntityStatus;
import hu.oroscom.c1801.enumeration.ServiceManagementNature;
import hu.oroscom.c1801.enumeration.ServiceManagementStatus;
import hu.oroscom.c1801.enumeration.StudyEntryType;
import hu.oroscom.c1801.util.MessagesUtil;
import org.apache.commons.lang.Validate;
import org.eclipse.persistence.descriptors.DescriptorEvent;
import org.eclipse.persistence.descriptors.DescriptorEventListener;
import org.joda.time.DateTime;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;
import org.vaadin.spring.i18n.I18N;

import javax.persistence.Embedded;
import javax.persistence.PostLoad;
import java.lang.reflect.Field;
import java.util.*;

/**
 * Created by Norbert on 2016. 01. 18..
 */
public class TrackChangesListener<T extends EntityBase> implements DescriptorEventListener {

	protected I18N i18n;
	protected static DateTimeFormatter dateTimeFormatter = DateTimeFormat.shortDateTime().withLocale(Locale.getDefault());

	@PostLoad
	private void postLoad(Object o) {

	}

	public void loadOldData(T entity) {
		if (i18n == null) {
			i18n = (I18N) Application.getApplicationContext().getBean("i18n");
		}

		if (!Editor.isEdited(entity)) {
			return;
		}

		HashMap<String, Object> oldData = new HashMap<>();

		collectOldValues(oldData, "", entity);

		Editor.setOldData(entity, oldData);
	}

	private void collectOldValues(HashMap<String, Object> oldData, String prefix, Object entity) {
		List<Field> historisedFields = getHistorisedFields(entity);

		for (Field field : historisedFields) {
			field.setAccessible(true);
			try {
				/*if (field.getAnnotation(ManyToMany.class) != null) {
					Collection<T> entityList = (Collection<T>) field.get(entity);
					for (T listEntity : entityList) {
						collectOldValues(oldData, prefix + field.getName() + ".", listEntity);
					}
				} else*/
				if (field.getAnnotation(Embedded.class) != null) {
					Object value = field.get(entity);
					if (value != null) {
						// recurse into the Embedded field
						collectOldValues(oldData, prefix + field.getName() + ".", value);
					}
				} else {
					oldData.put(prefix + field.getName(), field.get(entity));
				}
			} catch (IllegalAccessException e) {
				throw new RuntimeException(e);
			}
		}
	}

	public void updateChangeLog(T entity) {
		if (!Editor.isEdited(entity)) {
			return;
		}

		try {
			Field changeLogField = getChangeLogField(entity);
			Validate.notNull(changeLogField, "could not find @ChangeLog field in Class " + entity.getClass().getName());

			StringBuilder changeLog = new StringBuilder();

			String changeLogValue = (String) changeLogField.get(entity);

			if (changeLogValue != null) {
				changeLog.append(changeLogValue).append("\n\n");
			}

			changeLog.append(i18n.get("TrackChanges.changedBy", Editor.getUser().getDisplayName(), dateTimeFormatter.print(new DateTime()))).append('\n');
//			changeLog.append(" on ").append((new Date()).toGMTString());

			boolean changed = false;

			changed = collectChanges(changeLog, Editor.getOldData(entity), "", entity);

			// update the changeLog field if a trackable change was detected
			if (changed) {
				changeLogField.set(entity, changeLog.toString());
				loadOldData(entity);
			}
		} catch (IllegalAccessException e) {
			throw new RuntimeException(e);
		}
	}

	private Field getChangeLogField(Object entity) {
		Field[] fields = entity.getClass().getDeclaredFields();

		for (Field field : fields) {
			if (field.getAnnotation(ChangeLog.class) != null) {
				field.setAccessible(true);
				return field;
			}
		}

		fields = entity.getClass().getSuperclass().getDeclaredFields();

		for (Field field : fields) {
			if (field.getAnnotation(ChangeLog.class) != null) {
				field.setAccessible(true);
				return field;
			}
		}

		return null;
	}

	private boolean collectChanges(StringBuilder changeLog, Map oldData, String prefix, Object entity) {
		if (oldData == null) {
			return false;
		}
		boolean changed = false;
		List<Field> historisedFields = getHistorisedFields(entity);
		for (Field field : historisedFields) {
			field.setAccessible(true);
			String fieldName = field.getName();
			try {
				if (field.getAnnotation(Embedded.class) != null) {
					Object value = field.get(entity);
					if (value != null) {
						// recurse into the Embedded field
						changed |= collectChanges(changeLog, oldData, prefix + field.getName() + ".", value);
					}
				} else {
					Object newValue = field.get(entity);
					Object oldValue = oldData.get(prefix + fieldName);
					changed |= addChangeLine(changeLog, prefix + fieldName, oldValue, newValue);
				}
			} catch (IllegalAccessException e) {
				throw new RuntimeException(e);
			}
		}
		return changed;
	}

	private boolean addChangeLine(StringBuilder changeLog, String fieldName, Object oldValue, Object newValue) {
		if (oldValue == null && newValue == null) {
			return false;
		}
		if (oldValue instanceof Collection && newValue instanceof Collection) {
			List<BaseEntity> oldCollection = new ArrayList((Collection<BaseEntity>) oldValue);
			List<BaseEntity> newCollection = new ArrayList((Collection<BaseEntity>) newValue);
			Collections.sort(oldCollection);
			Collections.sort(newCollection);
			if (oldCollection.equals(newCollection)) {
				return false;
			}
		} else if (oldValue != null && oldValue.equals(newValue) && !(oldValue instanceof BaseEntity)) {
			// no change
			return false;
		} else if (oldValue instanceof BaseEntity) {
			BaseEntity oldBaseEntity = (BaseEntity) oldValue;
			BaseEntity newBaseEntity = (BaseEntity) newValue;

			if (oldBaseEntity.equals(newBaseEntity)) {
				return false;
			}
		}

		String oldValueString = oldValue != null ? oldValue.toString() : "";
		String newValueString = newValue != null ? newValue.toString() : "";

		boolean isChanged = false;
		if (oldValue instanceof Collection && newValue instanceof Collection) {
			Collection<Object> oldValueCollection = (Collection<Object>) oldValue;
			Collection<Object> newValueCollection = (Collection<Object>) newValue;

			for (Object newEntity : newValueCollection) {
				if (!oldValueCollection.contains(newEntity)) {
					isChanged = true;
					break;
				}
			}

			if (isChanged) {
				oldValueString = "";
				newValueString = "";

				newValueString = newValueString.replaceFirst(", ", "");
			}
		} else if (oldValue instanceof BaseEntity || newValue instanceof BaseEntity) {
			if (oldValue instanceof User || newValue instanceof User) {
				User oldUser = (User) oldValue;
				User newUser = (User) newValue;
				oldValueString = oldUser.getDisplayName();
				newValueString = newUser.getDisplayName();
			}
		} else if (oldValue instanceof EntityStatus || newValue instanceof EntityStatus) {
			EntityStatus oldEntityStatus = (EntityStatus) oldValue;
			EntityStatus newEntityStatus = (EntityStatus) newValue;
			oldValueString = oldEntityStatus != null ? oldEntityStatus.getCaption() : "";
			newValueString = newEntityStatus != null ? newEntityStatus.getCaption() : "";
		} else if (oldValue instanceof ServiceManagementNature || newValue instanceof ServiceManagementNature) {
			ServiceManagementNature oldServiceManagementNature = (ServiceManagementNature) oldValue;
			ServiceManagementNature newServiceManagementNature = (ServiceManagementNature) newValue;
			oldValueString = oldServiceManagementNature != null ? oldServiceManagementNature.getCaption() : "";
			newValueString = newServiceManagementNature != null ? newServiceManagementNature.getCaption() : "";
		} else if (oldValue instanceof ServiceManagementStatus || newValue instanceof ServiceManagementStatus) {
			ServiceManagementStatus oldServiceManagementStatus = (ServiceManagementStatus) oldValue;
			ServiceManagementStatus newServiceManagementStatus = (ServiceManagementStatus) newValue;
			oldValueString = oldServiceManagementStatus != null ? oldServiceManagementStatus.getCaption() : "";
			newValueString = newServiceManagementStatus != null ? newServiceManagementStatus.getCaption() : "";
		} else if (oldValue instanceof StudyEntryType || newValue instanceof StudyEntryType) {
			StudyEntryType oldStudyEntryType = (StudyEntryType) oldValue;
			StudyEntryType newStudyEntryType = (StudyEntryType) newValue;
			oldValueString = oldStudyEntryType != null ? oldStudyEntryType.getCaption() : "";
			newValueString = newStudyEntryType != null ? newStudyEntryType.getCaption() : "";
		} else if (oldValue instanceof Boolean || newValue instanceof Boolean) {
			oldValueString = oldValue != null ? i18n.get(oldValue.toString()) : "";
			newValueString = newValue != null ? i18n.get(newValue.toString()) : "";
		}

		String changeLine = MessagesUtil.get(i18n, "TrackChanges." + fieldName) + " " + i18n.get("TrackChanges.change", oldValueString, newValueString) + "\n";

		changeLog.append(changeLine);
		return true;
	}

	private List<Field> getHistorisedFields(Object entity) {
		Field[] fields = entity.getClass().getDeclaredFields();
		List<Field> histFields = new ArrayList<>();

		for (Field field : fields) {
			if (field.getAnnotation(TrackChanges.class) != null) {
				histFields.add(field);
			}
		}

		fields = entity.getClass().getSuperclass().getDeclaredFields();

		for (Field field : fields) {
			if (field.getAnnotation(TrackChanges.class) != null) {
				histFields.add(field);
			}
		}

		return histFields;
	}

	@Override
	public void aboutToDelete(DescriptorEvent descriptorEvent) {

	}

	@Override
	public void aboutToInsert(DescriptorEvent descriptorEvent) {

	}

	@Override
	public void aboutToUpdate(DescriptorEvent descriptorEvent) {

	}

	@Override
	public boolean isOverriddenEvent(DescriptorEvent descriptorEvent, Vector vector) {
		return false;
	}

	@Override
	public void postBuild(DescriptorEvent descriptorEvent) {
		T entity = (T) descriptorEvent.getObject();
		try {
			if (Editor.getOldData(entity) == null) {
				loadOldData(entity);
			}
		} catch (RuntimeException e) {

		}
	}

	@Override
	public void postClone(DescriptorEvent descriptorEvent) {
		T entity = (T) descriptorEvent.getObject();
		try {
			if (Editor.getOldData(entity) == null) {
				loadOldData(entity);
			}
		} catch (RuntimeException e) {

		}
	}

	@Override
	public void postDelete(DescriptorEvent descriptorEvent) {

	}

	@Override
	public void postInsert(DescriptorEvent descriptorEvent) {

	}

	@Override
	public void postMerge(DescriptorEvent descriptorEvent) {

	}

	@Override
	public void postRefresh(DescriptorEvent descriptorEvent) {

	}

	@Override
	public void postUpdate(DescriptorEvent descriptorEvent) {

	}

	@Override
	public void postWrite(DescriptorEvent descriptorEvent) {

	}

	@Override
	public void preDelete(DescriptorEvent descriptorEvent) {

	}

	@Override
	public void preInsert(DescriptorEvent descriptorEvent) {

	}

	@Override
	public void prePersist(DescriptorEvent descriptorEvent) {

	}

	@Override
	public void preRemove(DescriptorEvent descriptorEvent) {

	}

	@Override
	public void preUpdate(DescriptorEvent descriptorEvent) {
		updateChangeLog((T) descriptorEvent.getObject());
	}

	@Override
	public void preUpdateWithChanges(DescriptorEvent descriptorEvent) {
//		updateChangeLog((T) descriptorEvent.getObject());
	}

	@Override
	public void preWrite(DescriptorEvent descriptorEvent) {

	}
}
