package hu.oroscom.c1801.domain;

import hu.oroscom.c1801.converter.JodaDateTimeConverter;
import hu.oroscom.c1801.enumeration.*;
import org.hibernate.validator.constraints.NotEmpty;
import org.joda.time.DateTime;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;

import javax.persistence.*;
import javax.validation.constraints.Size;
import java.util.*;

@Entity
@Inheritance(strategy = InheritanceType.SINGLE_TABLE)
public class User extends BaseEntity implements UserDetails {
	@NotEmpty
	@Size(min = 2, max = 255)
	private String firstName;

	@NotEmpty
	@Size(min = 2, max = 255)
	private String lastName;

	@NotEmpty
	@Column(nullable = false)
	private String userName; //=userName

	@Size(min = 1, max = 10)
	private String shortName;

	@Size(max = 255)
	private String company;

	@NotEmpty
	@Column(nullable = false)
	private String password;

	@ManyToOne
	private Role role;

	@Transient
	private Set<PermissionName> permissions;

	@Transient
	private String displayName;

	@Size(max = 255)
	private String shortDescription;

	@Size(max = 255)
	private String description;

	@Column(columnDefinition = "TIMESTAMP NULL DEFAULT NULL")
	@org.eclipse.persistence.annotations.Converter(name = "dateTimeConverter", converterClass = JodaDateTimeConverter.class)
	@org.eclipse.persistence.annotations.Convert("dateTimeConverter")
	private DateTime lastLoginDate;

	@OneToMany(mappedBy = "user", cascade = CascadeType.ALL, orphanRemoval = true)
	private Set<GridSetting> gridSettings;

	@ManyToOne
	private Customer customer;

	@Enumerated
	private EntryPoint entryPoint = EntryPoint.DASHBOARD_VIEW;

	@OneToMany(mappedBy = "user")
	private List<ServiceManagement> serviceManagements;

	@Enumerated
	private Venue venue;

	@ManyToMany(mappedBy = "users")
	private Set<ServiceGroup> serviceGroups;

	private String enabledGlobalFilters = "OWN";

	@Enumerated
	private GlobalFilter defaultGlobalFilter;

	public String getFirstName() {
		return firstName;
	}

	public void setFirstName(String name) {
		this.firstName = name;
	}

	public String getUserName() {
		return userName;
	}

	public void setUserName(String userName) {
		this.userName = userName;
	}

	@Override
	public Collection<? extends GrantedAuthority> getAuthorities() {
		Collection<GrantedAuthority> authorities = new ArrayList<>();
		SimpleGrantedAuthority authority = new SimpleGrantedAuthority(this.getRole().getRoleName().toString());
		authorities.add(authority);
		return authorities;
	}

	public Set<PermissionName> getPermissions() {
		return permissions;
	}

	public void setPermissions(Set<PermissionName> permissions) {
		this.permissions = permissions;
	}

	@Override
	public String getPassword() {
		return password;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	@Override
	public String getUsername() {
		return getUserName();
	}

	@Override
	public boolean isAccountNonExpired() {
		return true;
	}

	@Override
	public boolean isAccountNonLocked() {
		return true;
	}

	@Override
	public boolean isCredentialsNonExpired() {
		return true;
	}

	@Override
	public boolean isEnabled() {
		return getStatus() == EntityStatus.ACTIVE;
	}

	public void setEnabled(boolean enabled) {
		setStatus(enabled ? EntityStatus.ACTIVE : EntityStatus.INACTIVE);
	}

	public boolean hasRole(RoleName roleName) {
		return getAuthorities().contains(new SimpleGrantedAuthority(roleName.toString()));
	}

	public Role getRole() {
		return role;
	}

	public void setRole(Role role) {
		this.role = role;
	}

	public String getLastName() {
		return lastName;
	}

	public void setLastName(String lastName) {
		this.lastName = lastName;
	}

	public String getShortName() {
		return shortName;
	}

	public void setShortName(String shortName) {
		this.shortName = shortName;
	}

	public String getShortDescription() {
		return shortDescription;
	}

	public void setShortDescription(String shortDescription) {
		this.shortDescription = shortDescription;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public void setDisplayName() {
		String fName = (this.getFirstName() != null) ? this.getFirstName() : "";
		String lName = (this.getLastName() != null) ? this.getLastName() : "";
		displayName = fName.trim() + " " + lName.trim();
		if (displayName.trim().length() == 0) {
			displayName = this.getUserName();
		}
	}

	public String getDisplayName() {
		return displayName;
	}

	public DateTime getLastLoginDate() {
		return lastLoginDate;
	}

	public void setLastLoginDate(DateTime lastLoginDate) {
		this.lastLoginDate = lastLoginDate;
	}

	public String getCompany() {
		return company;
	}

	public void setCompany(String company) {
		this.company = company;
	}

	public Set<GridSetting> getGridSettings() {
		return gridSettings;
	}

	public void setGridSettings(Set<GridSetting> gridSettings) {
		this.gridSettings = gridSettings;
	}

	public Customer getCustomer() {
		return customer;
	}

	public void setCustomer(Customer customer) {
		this.customer = customer;
	}

	public EntryPoint getEntryPoint() {
		return entryPoint;
	}

	public void setEntryPoint(EntryPoint entryPoint) {
		this.entryPoint = entryPoint;
	}

	public List<ServiceManagement> getServiceManagements() {
		return serviceManagements;
	}

	public void setServiceManagements(List<ServiceManagement> serviceManagements) {
		this.serviceManagements = serviceManagements;
	}

	public Venue getVenue() {
		return venue;
	}

	public void setVenue(Venue venue) {
		this.venue = venue;
	}

	public Set<ServiceGroup> getServiceGroups() {
		return serviceGroups;
	}

	public void setServiceGroups(Set<ServiceGroup> serviceGroups) {
		this.serviceGroups = serviceGroups;
	}

	public String getEnabledGlobalFilters() {
		return enabledGlobalFilters;
	}

	public void setEnabledGlobalFilters(String enabledGlobalFilters) {
		this.enabledGlobalFilters = enabledGlobalFilters;
	}

	public GlobalFilter getDefaultGlobalFilter() {
		return defaultGlobalFilter;
	}

	public void setDefaultGlobalFilter(GlobalFilter defaultGlobalFilter) {
		this.defaultGlobalFilter = defaultGlobalFilter;
	}

	@PostLoad
	public void postLoad() {
		setDisplayName();
		initPermissions();
	}

	private void initPermissions() {
		if (permissions == null) {
			permissions = new HashSet<>();
			for (Permission permission : this.getRole().getPermissions()) {
				permissions.add(PermissionName.fromString(permission.getAuthority()));
			}
		}
	}
}
