package hu.oroscom.c1801.service;

import hu.oroscom.c1801.domain.Complaint;
import hu.oroscom.c1801.domain.Patient;
import hu.oroscom.c1801.domain.filter.ComplaintFilter;
import hu.oroscom.c1801.enumeration.PermissionName;
import hu.oroscom.c1801.jparepository.ComplaintRepository;
import hu.oroscom.c1801.util.FilterItemBean;
import hu.oroscom.c1801.util.FilterUtil;
import org.joda.time.DateTime;
import org.springframework.stereotype.Service;

import javax.inject.Inject;
import javax.persistence.EntityManager;
import javax.persistence.TypedQuery;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

@Service
public class ComplaintService extends BaseService<Complaint> {

	@Inject
	ComplaintRepository complaintRepository;

	@Inject
	UserService userService;

	@Override
	public Complaint findOne(Long id) {
		return complaintRepository.findOne(id);
	}

	@Override
	public Complaint save(Complaint entity) {
		return complaintRepository.save(entity);
	}

	@Override
	public List<Complaint> save(Iterable<Complaint> entities) {
		return complaintRepository.save(entities);
	}

	@Override
	public List<Complaint> findAll() {
		return complaintRepository.findAll();
	}

	@Override
	public void delete(Complaint entity) {
		complaintRepository.delete(entity);
	}

	@Override
	public void delete(Long id) {
		complaintRepository.delete(id);
	}

	@Override
	public void deleteAll() {
		complaintRepository.deleteAll();
	}

	public boolean userHasPermission(PermissionName... permissionsNames) {
		return userService.userHasPermission(permissionsNames);
	}

	public List<Complaint> findAllByPatient(Patient patient) {
		return complaintRepository.findAllByPatient(patient);
	}

	public List<Complaint> findAllWithFilter(ComplaintFilter complaintFilter) {
		return queryBuilder(complaintFilter).getResultList();
	}

	private TypedQuery<Complaint> queryBuilder(ComplaintFilter complaintFilter) {
		EntityManager em = complaintRepository.getEntityManager();

		String queryString = "SELECT DISTINCT c FROM Complaint c";
		String joinString = "";
		String whereString = " WHERE c.patient = :patient";

		if (complaintFilter.getQuickSearch() != null && !complaintFilter.getQuickSearch().isEmpty()) {
			whereString += " AND (LOWER(c.description) LIKE :description)";
		}

		if (complaintFilter.getIds() != null && !complaintFilter.getIds().isEmpty()) {
			whereString += " AND c.id IN :ids";
		}

		if (complaintFilter.getDescription() != null && !complaintFilter.getDescription().isEmpty()) {
			whereString += getFilterItemCondition("c.description", complaintFilter.getDescription());
		}

		if (complaintFilter.getDateTime() != null && !complaintFilter.getDateTime().isEmpty()) {
			whereString += getFilterItemCondition("c.dateTime", complaintFilter.getDateTime(), DateTime.class, "time");
		}

		queryString += joinString + whereString;

		TypedQuery<Complaint> typedQuery = em.createQuery(queryString, Complaint.class);
		typedQuery.setHint("eclipselink.read-only", "true");
		typedQuery.setParameter("patient", complaintFilter.getPatient());

		if (complaintFilter.getQuickSearch() != null && !complaintFilter.getQuickSearch().isEmpty()) {
			typedQuery.setParameter("description", "%" + complaintFilter.getQuickSearch().toLowerCase() + "%");
		}

		if (complaintFilter.getIds() != null && !complaintFilter.getIds().isEmpty()) {
			Set<Long> entityIds = new HashSet<>();
			for (String filterItem : complaintFilter.getIds()) {
				FilterItemBean filterItemBean = FilterUtil.getFilterItem(filterItem);
				entityIds.add(Long.valueOf(filterItemBean.getFilterValue()));
			}

			typedQuery.setParameter("ids", entityIds);
		}

		if (complaintFilter.getDateTime() != null && !complaintFilter.getDateTime().isEmpty()) {
			setupDateTimeParameters(typedQuery, complaintFilter.getDateTime(), "time");
		}

		return typedQuery;
	}
}
