package hu.oroscom.c1801.service;

import hu.oroscom.c1801.domain.Customer;
import hu.oroscom.c1801.domain.Permission;
import hu.oroscom.c1801.domain.Role;
import hu.oroscom.c1801.enumeration.PermissionName;
import hu.oroscom.c1801.enumeration.RoleName;
import hu.oroscom.c1801.jparepository.RoleRepository;
import hu.oroscom.c1801.util.UserUtil;
import org.springframework.stereotype.Service;
import org.vaadin.spring.i18n.I18N;

import javax.inject.Inject;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.List;

@Service
public class RoleService extends BaseService<Role> {

	@Inject
	RoleRepository roleRepository;

	@Inject
	UserService userService;

	@Inject
	PermissionService permissionService;

	@Inject
	I18N i18n;

	@Override
	public Role findOne(Long id) {
		return roleRepository.findOne(id);
	}

	@Override
	public List<Role> findAll() {
		return roleRepository.findAll();
	}

	@Override
	public Role save(Role entity) {
		return roleRepository.save(entity);
	}

	@Override
	public List<Role> save(Iterable<Role> entities) {
		return roleRepository.save(entities);
	}

	@Override
	public void delete(Role entity) {
		roleRepository.delete(entity);
	}

	@Override
	public void delete(Long id) {
		roleRepository.delete(id);
	}

	@Override
	public void deleteAll() {
		roleRepository.deleteAll();
	}

	public Role merge(Role role) {
		return roleRepository.merge(role);
	}

	public List<Role> findAllWithoutRole(RoleName... roleName) {
		return findAllWithoutRole(UserUtil.getCustomer(), roleName);
	}

	private List<Role> findAllWithoutRole(Customer customer, RoleName... roleName) {
		return roleRepository.findAllWithoutRole(customer, roleName);
	}

	public Role findSuperAdmin() {
		return roleRepository.findSuperAdmin(RoleName.ROLE_SUPER_ADMIN.toString());
	}

	public List<Role> findAllWithRole(Customer customer, RoleName... roleName) {
		return roleRepository.findAllWithRole(customer, roleName);
	}

	public List<Role> findAllBySuperAdmin() {
		return roleRepository.findAllBySuperAdmin();
	}

	public List<Role> findAllByCustomer() {
		return findAllByCustomer(UserUtil.getCustomer());
	}

	public List<Role> findAllByCustomer(Customer customer) {
		return roleRepository.findAllByCustomer(customer);
	}

	public boolean userHasPermission(PermissionName... permissionsNames) {
		return userService.userHasPermission(permissionsNames);
	}

	public Role findRoleByName(RoleName roleName) {
		return findRoleByName(roleName, UserUtil.getCustomer());
	}

	public Role findRoleByName(RoleName roleName, Customer customer) {
		return roleRepository.findRoleByName(roleName.toString(), customer);
	}

	public void createDefaultRoles(Customer customer) {
		boolean rolesWasEmpty = false;
		if (findAllByCustomer(customer).isEmpty()) {
			rolesWasEmpty = true;
			for (RoleName roleName : RoleName.values()) {
				Role role = new Role(roleName, customer);
				merge(role);
			}
		}

		if (permissionService.findAll().isEmpty()) {
			for (PermissionName permissionName : PermissionName.values()) {
				Permission permission = new Permission(permissionName);
				permissionService.merge(permission);
			}
		}

		if (rolesWasEmpty) {
			assignPermissionsWithRoles(customer);
		}
	}


	private void assignPermissionsWithRoles(Customer customer) {
		BufferedReader bufferedReader = null;
		String SEPARATOR = ";";
		String csvFile = "rolePermisssion.csv";
		String line;
		String[] roles = new String[RoleName.values().length];

		try {
			InputStream inputStream = RoleService.class.getClassLoader().getResourceAsStream(csvFile);
			bufferedReader = new BufferedReader(new InputStreamReader(inputStream));
			while ((line = bufferedReader.readLine()) != null) {
				String[] columns = line.split(SEPARATOR, RoleName.values().length + 1);
				if (roles[0] == null) {
					roles = columns.clone();
				}
				for (int i = 1; i < columns.length; i++) {
					if (columns[i].toLowerCase().equals("x")) {
						Role role = findRoleByName(RoleName.fromString(roles[i]), customer);
						if (role == null) {
							continue;
						}
						Permission permission = permissionService.findPermissionByName(PermissionName.fromString(columns[0]));
						role.addPermission(permission);
						merge(role);
					}
				}
			}
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			if (bufferedReader != null) {
				try {
					bufferedReader.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
	}

	public void duplicateRole(Long id) {
		Role origRole = findOne(id);

		Role role = new Role(origRole.getCaption() + i18n.get("RoleService.copy"), origRole.getCustomer());
		role.setPermissions(origRole.getPermissions());

		merge(role);
	}
}
