package hu.oroscom.c1801.service;

import hu.oroscom.c1801.bean.BroadcastBean;
import hu.oroscom.c1801.broadcaster.Broadcaster;
import hu.oroscom.c1801.domain.Doctor;
import hu.oroscom.c1801.domain.Patient;
import hu.oroscom.c1801.domain.UserComment;
import hu.oroscom.c1801.domain.filter.UserCommentFilter;
import hu.oroscom.c1801.enumeration.BroadcastType;
import hu.oroscom.c1801.enumeration.PermissionName;
import hu.oroscom.c1801.enumeration.UserCommentType;
import hu.oroscom.c1801.jparepository.UserCommentRepository;
import hu.oroscom.c1801.util.FilterItemBean;
import hu.oroscom.c1801.util.FilterUtil;
import hu.oroscom.c1801.util.UserUtil;
import org.joda.time.DateTime;
import org.springframework.stereotype.Service;

import javax.inject.Inject;
import javax.persistence.EntityManager;
import javax.persistence.TypedQuery;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * Created by Norbert on 2014.10.29..
 */
@Service
public class UserCommentService extends BaseService<UserComment> {

	@Inject
	private UserCommentRepository userCommentRepository;

	@Inject
	private UserService userService;

	@Override
	public UserComment findOne(Long id) {
		return userCommentRepository.findOne(id);
	}

	@Override
	public UserComment save(UserComment entity) {
		UserComment userComment = userCommentRepository.save(entity);
		Broadcaster.broadcast(new BroadcastBean(BroadcastType.MESSAGE, userComment));
		return userComment;
	}

	@Override
	public List<UserComment> save(Iterable<UserComment> entities) {
		return userCommentRepository.save(entities);
	}

	@Override
	public List<UserComment> findAll() {
		return userCommentRepository.findAll();
	}

	@Override
	public void delete(UserComment entity) {
		userCommentRepository.delete(entity);
		Broadcaster.broadcast(new BroadcastBean(BroadcastType.MESSAGE, entity));
	}

	@Override
	public void delete(Long id) {
		userCommentRepository.delete(id);
	}

	@Override
	public void deleteAll() {
		userCommentRepository.deleteAll();
	}

	public boolean userHasPermission(PermissionName... permissionsNames) {
		return userService.userHasPermission(permissionsNames);
	}

	public int findUnmanagedCommentCount() {
		if (UserUtil.isAssistant()) {
			return userCommentRepository.findAssistantUnmanagedCommentCount(UserUtil.getUser().getVenue(), UserUtil.getCustomer());
		} else if (UserUtil.isDoctor()) {
			return userCommentRepository.findDoctorUnmanagedCommentCount(UserCommentType.ASSIGNED_TO_DOCTOR, (Doctor) UserUtil.getUser(), UserUtil.getCustomer());
		} else if (UserUtil.isPatient()) {
			return userCommentRepository.findPatientUnseenAnsweredCommentCount(UserCommentType.ANSWERED, UserCommentType.BROADCAST, (Patient) UserUtil.getUser(), UserUtil.getCustomer());
		}
		return 0;
	}

	public List<UserComment> findAllWithFilter(UserCommentFilter userCommentFilter) {
		return queryBuilder(userCommentFilter).getResultList();
	}

	private TypedQuery<UserComment> queryBuilder(UserCommentFilter userCommentFilter) {
		EntityManager em = userCommentRepository.getEntityManager();

		String queryString = "SELECT DISTINCT uc FROM UserComment uc";
		String joinString = "";
		String whereString = " WHERE uc.customer = :customer";
		String orderByString = " ORDER BY uc.insertedAt DESC";

		if (UserUtil.isAssistant()) {
			joinString += " INNER JOIN uc.patients p";
			whereString += " AND p.venue = :venue";
		} else if (UserUtil.isDoctor()) {
			whereString += " AND uc.doctor = :doctor AND uc.userCommentType IS NOT NULL";
		} else if (UserUtil.isPatient()) {
			whereString += " AND uc.patients IN(:patient)";
		}

		if (userCommentFilter.getQuickSearch() != null && !userCommentFilter.getQuickSearch().isEmpty()) {
			whereString += " AND (LOWER(uc.question) LIKE :question)";
		}

		if (userCommentFilter.getIds() != null && !userCommentFilter.getIds().isEmpty()) {
			whereString += " AND uc.id IN :ids";
		}

		if (userCommentFilter.getUserCommentType() != null && !userCommentFilter.getUserCommentType().isEmpty()) {
			whereString += getFilterItemCondition("uc.userCommentType", userCommentFilter.getUserCommentType(), UserCommentType.class);
		}

		if (userCommentFilter.getCommentInsertedAt() != null && !userCommentFilter.getCommentInsertedAt().isEmpty()) {
			whereString += getFilterItemCondition("uc.insertedAt", userCommentFilter.getCommentInsertedAt(), DateTime.class, "insertedAt");
		}

		queryString += joinString + whereString + orderByString;

		TypedQuery<UserComment> typedQuery = em.createQuery(queryString, UserComment.class);
		typedQuery.setHint("eclipselink.read-only", "true");
		typedQuery.setParameter("customer", UserUtil.getCustomer());
		if (UserUtil.isAssistant()) {
			typedQuery.setParameter("venue", UserUtil.getUser().getVenue());
		} else if (UserUtil.isDoctor()) {
			typedQuery.setParameter("doctor", UserUtil.getUser());
		} else if (UserUtil.isPatient()) {
			typedQuery.setParameter("patient", UserUtil.getUser());
		}

		if (userCommentFilter.getQuickSearch() != null && !userCommentFilter.getQuickSearch().isEmpty()) {
			typedQuery.setParameter("question", "%" + userCommentFilter.getQuickSearch().toLowerCase() + "%");
		}

		if (userCommentFilter.getIds() != null && !userCommentFilter.getIds().isEmpty()) {
			Set<Long> entityIds = new HashSet<>();
			for (String filterItem : userCommentFilter.getIds()) {
				FilterItemBean filterItemBean = FilterUtil.getFilterItem(filterItem);
				entityIds.add(Long.valueOf(filterItemBean.getFilterValue()));
			}

			typedQuery.setParameter("ids", entityIds);
		}

		if (userCommentFilter.getCommentInsertedAt() != null && !userCommentFilter.getCommentInsertedAt().isEmpty()) {
			setupDateTimeParameters(typedQuery, userCommentFilter.getCommentInsertedAt(), "insertedAt");
		}

		if (userCommentFilter.getUserCommentType() != null && !userCommentFilter.getUserCommentType().isEmpty()) {
			int index = 0;
			for (String filterItem : userCommentFilter.getUserCommentType()) {
				FilterItemBean filterItemBean = FilterUtil.getFilterItem(filterItem);
				UserCommentType filterValue = UserCommentType.valueOf(filterItemBean.getFilterValue());
				typedQuery.setParameter("userCommentType" + index, filterValue);
				index++;
			}
		}

		return typedQuery;
	}
}
