package hu.oroscom.c1801.service;

import com.vaadin.server.VaadinSession;
import hu.oroscom.c1801.domain.Customer;
import hu.oroscom.c1801.domain.Role;
import hu.oroscom.c1801.domain.Setting;
import hu.oroscom.c1801.domain.User;
import hu.oroscom.c1801.enumeration.EntityStatus;
import hu.oroscom.c1801.enumeration.PermissionName;
import hu.oroscom.c1801.enumeration.RoleName;
import hu.oroscom.c1801.enumeration.SettingName;
import hu.oroscom.c1801.jparepository.UserRepository;
import hu.oroscom.c1801.util.UserUtil;
import org.joda.time.DateTime;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.stereotype.Service;

import javax.inject.Inject;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

@Service
public class UserService extends BaseService<User> {

	@Inject
	UserRepository userRepository;

	@Inject
	SettingService settingService;

	@Inject
	RoleService roleService;

	public User findUserByUsername(String userName) {
		return userRepository.findUserByUserName(userName);
	}

	public User createUser(User user) {
		return userRepository.save(user);
	}

	@PreAuthorize("hasPermission(authentication, 'CREATE_USER')")
	public User createUser2(User user) {
		return userRepository.save(user);
	}

	public void delete(Long id) {
		userRepository.delete(id);
	}

	@Override
	public void deleteAll() {
		userRepository.deleteAll();
	}

	public User find(Long id) {
		return userRepository.findOne(id);
	}

	@Override
	public List<User> save(Iterable entities) {
		return userRepository.save(entities);
	}

	@Override
	public User findOne(Long id) {
		return userRepository.findOne(id);
	}

	@Override
	public User save(User entity) {
		User user = userRepository.save(entity);
		if (VaadinSession.getCurrent() != null) {
			User sessionUser = UserUtil.getUser();
			if (user.getId().equals(sessionUser.getId())) {
				VaadinSession.getCurrent().setAttribute(User.class, user);
			}
		}
		return user;
	}

	public User merge(User entity) {
		return userRepository.merge(entity);
	}

	@Override
	public List<User> findAll() {
		return userRepository.findAll();
	}

	@Override
	public void delete(User entity) {
		userRepository.delete(entity);
	}

	public Setting getSettingByName(SettingName settingName) {
		return settingService.findSetting(settingName);
	}

	public boolean userHasPermission(PermissionName... permissionsNames) {
		User user = UserUtil.getUser();
		for (PermissionName permissionName : permissionsNames) {
			return user.getPermissions().contains(permissionName);
		}
		return false;
	}

	public boolean userHasAllPermission(PermissionName... permissionsNames) {
		User user = UserUtil.getUser();
		Set<PermissionName> permissions = new HashSet<>(Arrays.asList(permissionsNames));
		return user.getPermissions().containsAll(permissions);
	}

	public boolean userHasAllPermission(User user, Set<PermissionName> permissions) {
		return user.getPermissions().containsAll(permissions);
	}

	public boolean userHasAnyPermission(User user, Set<PermissionName> permissions) {
		for (PermissionName permission : permissions) {
			if (user.getPermissions().contains(permission)) {
				return true;
			}
		}
		return false;
	}

	public List<User> findAllByRole(Role role) {
		return userRepository.findAllByRole(role);
	}

	public List<User> findAllByRoleName(RoleName roleName) {
		Role role = roleService.findRoleByName(roleName);
		return userRepository.findAllByRole(role);
	}
	public User findByEmailAndPassword(String username, String password) {
		return userRepository.findByUserNameAndPassword(username, password, EntityStatus.ACTIVE);
	}

	public User userLoggedIn(User user) {
		if (VaadinSession.getCurrent() == null) {
			return user;
		}
		DateTime lastLoginDateTime = user.getLastLoginDate();
		VaadinSession.getCurrent().setAttribute("lastLogin", lastLoginDateTime);
		user.setLastLoginDate(new DateTime());
		return merge(user);
	}

	public List<User> findAllApiUsers(RoleName roleName) {
		return findAllByRoleName(roleName);
	}

	public List<Role> findAllWithRole(Customer customer, RoleName... roleName) {
		return roleService.findAllWithRole(customer, roleName);
	}

	public void addUserToApiSettings(User user) {
//		userServiceService.addUserToApiSettings(user);
	}

	public void removeUserToApiSettings(User user) {
//		userServiceService.removeUserToApiSettings(user);
	}

	public List<User> findAllByCustomer() {
		return userRepository.findAllByCustomer(UserUtil.getCustomer(), EntityStatus.ACTIVE);
	}

	public List<User> findAllByCustomer(RoleName roleName) {
		return userRepository.findAllByCustomer(UserUtil.getCustomer(), EntityStatus.ACTIVE);
	}

	public Role findRoleByRoleName(RoleName roleName) {
		return roleService.findRoleByName(roleName);
	}
}
