package hu.oroscom.c1801.view;

import com.vaadin.data.Item;
import com.vaadin.data.Property;
import com.vaadin.server.VaadinSession;
import com.vaadin.ui.*;
import com.vaadin.ui.themes.ValoTheme;
import hu.oroscom.c1801.NeededAnyPermission;
import hu.oroscom.c1801.component.FilterInfo;
import hu.oroscom.c1801.component.cell.EditDateTimeFormatCell;
import hu.oroscom.c1801.component.cell.RemoveCell;
import hu.oroscom.c1801.component.form.ComplaintForm;
import hu.oroscom.c1801.component.table.BaseTable;
import hu.oroscom.c1801.component.windowed.FilterPanel;
import hu.oroscom.c1801.domain.Complaint;
import hu.oroscom.c1801.domain.Patient;
import hu.oroscom.c1801.domain.filter.ComplaintFilter;
import hu.oroscom.c1801.enumeration.Action;
import hu.oroscom.c1801.enumeration.BulkAction;
import hu.oroscom.c1801.enumeration.FilterCategory;
import hu.oroscom.c1801.enumeration.PermissionName;
import hu.oroscom.c1801.service.ComplaintService;
import hu.oroscom.c1801.service.ExcelExportLogService;
import hu.oroscom.c1801.service.FilterService;
import hu.oroscom.c1801.service.IntolerantMaterialService;
import hu.oroscom.c1801.util.EventBusAction;
import hu.oroscom.c1801.util.FilterUtil;
import hu.oroscom.c1801.util.LoggingExcelExport;
import hu.oroscom.c1801.util.UserUtil;
import org.joda.time.DateTime;
import org.vaadin.spring.annotation.VaadinUIScope;
import org.vaadin.spring.events.annotation.EventBusListenerMethod;
import org.vaadin.spring.navigator.annotation.VaadinView;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

@VaadinUIScope
@VaadinView(name = ComplaintsView.NAME)
@NeededAnyPermission(values = {
		PermissionName.PERM_COMPLAINT_SELECT_ANY
})
public class ComplaintsView extends GridView<Complaint, ComplaintService, ComplaintForm> {
	public static final String NAME = "complaints";
	private final List<Object> visibleColumns = new ArrayList<>(Arrays.asList(new Object[]{"dateTime", "description", "remove"}));
	private Patient patient;
	private ComboBox otherComboBox = new ComboBox();
	private ComboBox filterComboBox = new ComboBox();
	private TextField quickSearchTextField = new TextField();
	private ComplaintFilter eatenFoodFilter = new ComplaintFilter();
	private VerticalLayout filterInfoVerticalLayout = new VerticalLayout();
	private FilterInfo filterInfo;

	@Inject
	private ComplaintService complaintService;

	@Inject
	private FilterService filterService;

	@Inject
	private ExcelExportLogService excelExportLogService;

	private final Property.ValueChangeListener filterComboBoxValueChangeListener = new Property.ValueChangeListener() {
		@Override
		public void valueChange(Property.ValueChangeEvent valueChangeEvent) {
			if ("showFilter".equals(valueChangeEvent.getProperty().getValue())) {
				FilterPanel filterPanel = new FilterPanel(eatenFoodFilter, filterService, complaintService);
				addContentToWindow(filterPanel);
			} else {
				eatenFoodFilter = (ComplaintFilter) valueChangeEvent.getProperty().getValue();
				refreshTable();
			}

			filterComboBox.removeValueChangeListener(filterComboBoxValueChangeListener);
			filterComboBox.setValue(null);
			filterComboBox.addValueChangeListener(filterComboBoxValueChangeListener);
		}
	};

	private final Property.ValueChangeListener quickSearchValueChangeListener = new Property.ValueChangeListener() {
		@Override
		public void valueChange(Property.ValueChangeEvent event) {
			String searchText = event.getProperty().getValue().toString();
			if (searchText.length() > 2) {
				eatenFoodFilter.setQuickSearch(searchText);
				eventBus.publish(this, new EventBusAction<>(Action.FILTER, eatenFoodFilter));
			} else {
				Notification.show(i18n.get("QuickSearch.minThreeCharacter"), Notification.Type.WARNING_MESSAGE);
			}
		}
	};

	private final Property.ValueChangeListener otherComboBoxValueChangeListener = new Property.ValueChangeListener() {
		@Override
		public void valueChange(Property.ValueChangeEvent valueChangeEvent) {
			final BulkAction bulkAction = (BulkAction) valueChangeEvent.getProperty().getValue();
			switch (bulkAction) {
				case EXPORT:
					LoggingExcelExport excelExport = new LoggingExcelExport(baseTable, i18n.get("ComplaintsView.heading"), i18n.get("ComplaintsView.heading"), "complaints.xls", false, excelExportLogService, i18n.get("ComplaintsView.heading"));
					excelExport.excludeCollapsedColumns();
					Iterator iterator = excelExport.getPropIds().iterator();

					while (iterator.hasNext()) {
						Object propId = iterator.next();
						if ("remove".equals(propId)) {
							iterator.remove();
						}
					}
					excelExport.export();
					break;
			}

			otherComboBox.removeValueChangeListener(otherComboBoxValueChangeListener);
			otherComboBox.setValue(null);
			otherComboBox.addValueChangeListener(otherComboBoxValueChangeListener);
		}
	};

	@Override
	protected void init() {
		super.defaultComponents(i18n.get("ComplaintsView.heading"));
		initTable();
	}

	@Override
	protected void customizeView() {
		if (!(UserUtil.getUser()  instanceof Patient)) {
			setBackButton(PatientsView.NAME);
		}

		Label expandingGap = new Label();
		expandingGap.setWidth("100%");

		getHeaderToolBarHorizontalLayout().setWidth(100, Unit.PERCENTAGE);
		getHeaderToolBarHorizontalLayout().setSpacing(true);
		getHeaderToolBarHorizontalLayout().addComponent(getQuickSearchComponent());
		getHeaderToolBarHorizontalLayout().addComponent(getFilterComboBox());
		getHeaderToolBarHorizontalLayout().addComponent(expandingGap);
		getHeaderToolBarHorizontalLayout().addComponent(getOtherComboBox());
		getHeaderToolBarHorizontalLayout().setExpandRatio(expandingGap, 1);

		if (complaintService.userHasPermission(PermissionName.PERM_COMPLAINT_CREATE_ANY)) {
			setAddButton(i18n.get("ComplaintsView.addButton"));
		}
	}

	@Override
	protected void registerColumns() {
		baseTable.addStyleName("complaintsView-view");
		baseTable.setHeight(100, Unit.PERCENTAGE);
		baseTable.addStyleName(ValoTheme.TABLE_SMALL);
		baseTable.setColumnCollapsingAllowed(true);
		baseTable.setColumnReorderingAllowed(true);

		addGeneratedColumn("dateTime", new EditDateTimeFormatCell(editClickListener));
		addGeneratedColumn("remove", new RemoveCell<>(IntolerantMaterialService.class));

		addContainerProperty("dateTime", DateTime.class, null);
		addContainerProperty("description", String.class, "");
		addContainerProperty("isRemoveCellEnabled", Boolean.class, false);
		addContainerProperty("isEditCellEnabled", Boolean.class, false);

		baseTable.setColumnHeader("dateTime", i18n.get("Complaint.dateTime"));
		baseTable.setColumnHeader("description", i18n.get("Complaint.description"));
		baseTable.setColumnHeader("remove", "");
	}

	@Override
	protected Complaint getFormEntity() {
		Complaint complaint = new Complaint();
		complaint.setPatient(patient);
		return complaint;
	}

	@Override
	protected void generateForm(Complaint tEntity) {
		form = new ComplaintForm(tEntity, true);
	}

	@Override
	protected void setGridDataSource() {
		setPatient();

		ComplaintFilter sessionStudyEntryFilter = VaadinSession.getCurrent().getAttribute(ComplaintFilter.class);
		if (sessionStudyEntryFilter != null) {
			eatenFoodFilter = sessionStudyEntryFilter;
			VaadinSession.getCurrent().setAttribute(ComplaintFilter.class, null);
		}

		if (FilterUtil.isFilterActive(eatenFoodFilter)) {
			try {
				filterInfoVerticalLayout.removeComponent(filterInfo);
			} catch (Exception e) {
			}
			filterInfo = new FilterInfo<>(eatenFoodFilter);
			filterInfoVerticalLayout.addComponent(filterInfo, 0);
		} else {
			try {
				quickSearchTextField.setValue("");
				filterInfoVerticalLayout.removeComponent(filterInfo);
			} catch (Exception e) {
			}
		}

		eatenFoodFilter.setPatient(patient);

		List<Complaint> complaints = complaintService.findAllWithFilter(eatenFoodFilter);
		if (filterInfo != null) {
			filterInfo.setEntityCount(complaints.size());
		}

		baseTable.setContainerDataSource(null);
		for (Complaint complaint : complaints) {
			Item item = indexedContainer.addItem(complaint.getId());
			item.getItemProperty("dateTime").setValue(complaint.getDateTime());
			item.getItemProperty("description").setValue(complaint.getDescription());
			if (complaintService.userHasPermission(PermissionName.PERM_COMPLAINT_DELETE_ANY)) {
				item.getItemProperty("isRemoveCellEnabled").setValue(Boolean.TRUE);
			}
			if (complaintService.userHasPermission(PermissionName.PERM_COMPLAINT_UPDATE_ANY)) {
				item.getItemProperty("isEditCellEnabled").setValue(Boolean.TRUE);
			}
		}
		baseTable.setContainerDataSource(indexedContainer);
		setVisibleColumns(visibleColumns.toArray());
	}

	@Override
	protected void setBaseTable() {
		baseTable = new BaseTable(null, indexedContainer);
		baseTable.setCacheRate(2);

		if (filterInfoVerticalLayout.getComponentCount() != 0) {
			filterInfoVerticalLayout = new VerticalLayout();
			eatenFoodFilter = new ComplaintFilter();
		}

		filterInfoVerticalLayout.setImmediate(true);
		filterInfoVerticalLayout.setSizeFull();
		filterInfoVerticalLayout.addComponent(baseTable);
		filterInfoVerticalLayout.setExpandRatio(baseTable, 1);
		addComponent(filterInfoVerticalLayout);
		setExpandRatio(filterInfoVerticalLayout, 1);
	}

	@Override
	public void windowClose(Window.CloseEvent closeEvent) {

	}

	private void setPatient() {
		if (UserUtil.getUser() instanceof Patient) {
			patient = (Patient) UserUtil.getUser();
		} else {
			if (VaadinSession.getCurrent().getAttribute(Patient.class) != null) {
				patient = VaadinSession.getCurrent().getAttribute(Patient.class);
			}
		}

		VaadinSession.getCurrent().setAttribute(Patient.class, null);

		if (!(UserUtil.getUser() instanceof Patient)) {
			h1.setValue(i18n.get("ComplaintsView.heading") + " (" + patient.getDisplayName() + ")");
		}
	}

	private Component getQuickSearchComponent() {
		quickSearchTextField.addStyleName(ValoTheme.TEXTFIELD_SMALL);
		quickSearchTextField.setInputPrompt(i18n.get("BaseForm.quickSearch"));
		quickSearchTextField.addValueChangeListener(quickSearchValueChangeListener);

		return quickSearchTextField;
	}

	private ComboBox getFilterComboBox() {
		filterComboBox.setWidth(90, Unit.PIXELS);
		filterComboBox.addStyleName(ValoTheme.COMBOBOX_SMALL);
		filterComboBox.setNullSelectionAllowed(false);
		filterComboBox.setTextInputAllowed(false);
		filterComboBox.setInputPrompt(i18n.get("BaseView.filterLabel"));
		setFilterComboBoxItems();
		filterComboBox.addValueChangeListener(filterComboBoxValueChangeListener);

		return filterComboBox;
	}

	private void setFilterComboBoxItems() {
		filterComboBox.removeAllItems();
		for (ComplaintFilter eatenFoodFilter : filterService.findAllByUser(UserUtil.getUser(), FilterCategory.COMPLAINT, ComplaintFilter.class)) {
			filterComboBox.addItem(eatenFoodFilter);
			filterComboBox.setItemCaption(eatenFoodFilter, eatenFoodFilter.getFilterName());
		}

		filterComboBox.addItem("showFilter");
		filterComboBox.setItemCaption("showFilter", i18n.get("BaseView.showFilterLabel"));
	}

	private ComboBox getOtherComboBox() {
		otherComboBox.setWidth(90, Unit.PIXELS);
		otherComboBox.addStyleName(ValoTheme.COMBOBOX_SMALL);
		otherComboBox.setNullSelectionAllowed(false);
		otherComboBox.setTextInputAllowed(false);
		otherComboBox.setInputPrompt(i18n.get("GridView.otherLabel"));

		otherComboBox.addItem(BulkAction.EXPORT);
		otherComboBox.setItemCaption(BulkAction.EXPORT, i18n.get("GridView.otherExportLabel"));

		otherComboBox.addValueChangeListener(otherComboBoxValueChangeListener);

		return otherComboBox;
	}

	@EventBusListenerMethod
	public void buttonClick(EventBusAction eventBusAction) {
		if (eventBusAction.getFilter() != null && ComplaintFilter.class.getName().equals(eventBusAction.getFilter().getClass().getName())) {
			if (eventBusAction.getAction().equals(Action.FILTER)) {
				eatenFoodFilter = (ComplaintFilter) eventBusAction.getFilter();
				refreshTable();
			} else if (eventBusAction.getAction().equals(Action.DELETE_FILTER)) {
				quickSearchTextField.removeValueChangeListener(quickSearchValueChangeListener);
				eatenFoodFilter = (ComplaintFilter) eventBusAction.getFilter();
				quickSearchTextField.setValue("");
				quickSearchTextField.addValueChangeListener(quickSearchValueChangeListener);
				refreshTable();
			} else if (eventBusAction.getAction().equals(Action.SAVE_FILTER)) {
				setFilterComboBoxItems();
			}
		}
	}
}
