package hu.oroscom.c1801.view;

import com.vaadin.data.Item;
import com.vaadin.data.Property;
import com.vaadin.server.VaadinSession;
import com.vaadin.ui.*;
import com.vaadin.ui.themes.ValoTheme;
import hu.oroscom.c1801.NeededAnyPermission;
import hu.oroscom.c1801.component.FilterInfo;
import hu.oroscom.c1801.component.cell.CountFilterCell;
import hu.oroscom.c1801.component.cell.EditCell;
import hu.oroscom.c1801.component.cell.RemoveCell;
import hu.oroscom.c1801.component.cell.StatusCell;
import hu.oroscom.c1801.component.form.DoctorForm;
import hu.oroscom.c1801.component.table.BaseTable;
import hu.oroscom.c1801.component.table.TableObject;
import hu.oroscom.c1801.component.windowed.FilterPanel;
import hu.oroscom.c1801.domain.Doctor;
import hu.oroscom.c1801.domain.Patient;
import hu.oroscom.c1801.domain.filter.DoctorFilter;
import hu.oroscom.c1801.domain.filter.PatientFilter;
import hu.oroscom.c1801.enumeration.*;
import hu.oroscom.c1801.service.DoctorService;
import hu.oroscom.c1801.service.ExcelExportLogService;
import hu.oroscom.c1801.service.FilterService;
import hu.oroscom.c1801.service.UserService;
import hu.oroscom.c1801.util.EventBusAction;
import hu.oroscom.c1801.util.FilterUtil;
import hu.oroscom.c1801.util.LoggingExcelExport;
import hu.oroscom.c1801.util.UserUtil;
import org.vaadin.spring.annotation.VaadinUIScope;
import org.vaadin.spring.events.annotation.EventBusListenerMethod;
import org.vaadin.spring.navigator.annotation.VaadinView;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

@VaadinUIScope
@VaadinView(name = DoctorsView.NAME)
@NeededAnyPermission(values = {
		PermissionName.PERM_DOCTOR_SELECT_ANY
})
public class DoctorsView extends GridView<Doctor, DoctorService, DoctorForm> {
	public static final String NAME = "doctors";
	private final List<Object> visibleColumns = new ArrayList<>(Arrays.asList(new Object[]{"displayName", "userName", "status", "patients", "remove"}));

	private ComboBox otherComboBox = new ComboBox();
	private ComboBox filterComboBox = new ComboBox();
	private TextField quickSearchTextField = new TextField();
	private DoctorFilter serviceGroupFilter = new DoctorFilter();
	private VerticalLayout filterInfoVerticalLayout = new VerticalLayout();
	private FilterInfo filterInfo;

	@Inject
	private DoctorService doctorService;

	@Inject
	private FilterService filterService;

	@Inject
	private ExcelExportLogService excelExportLogService;

	private final Property.ValueChangeListener filterComboBoxValueChangeListener = new Property.ValueChangeListener() {
		@Override
		public void valueChange(Property.ValueChangeEvent valueChangeEvent) {
			if ("showFilter".equals(valueChangeEvent.getProperty().getValue())) {
				FilterPanel filterPanel = new FilterPanel(serviceGroupFilter, filterService, doctorService);
				addContentToWindow(filterPanel);
			} else {
				serviceGroupFilter = (DoctorFilter) valueChangeEvent.getProperty().getValue();
				refreshTable();
			}

			filterComboBox.removeValueChangeListener(filterComboBoxValueChangeListener);
			filterComboBox.setValue(null);
			filterComboBox.addValueChangeListener(filterComboBoxValueChangeListener);
		}
	};

	private final Property.ValueChangeListener quickSearchValueChangeListener = new Property.ValueChangeListener() {
		@Override
		public void valueChange(Property.ValueChangeEvent event) {
			String searchText = event.getProperty().getValue().toString();
			if (searchText.length() > 2) {
				serviceGroupFilter.setQuickSearch(searchText);
				eventBus.publish(this, new EventBusAction<>(Action.FILTER, serviceGroupFilter));
			} else {
				Notification.show(i18n.get("QuickSearch.minThreeCharacter"), Notification.Type.WARNING_MESSAGE);
			}
		}
	};

	private final Property.ValueChangeListener otherComboBoxValueChangeListener = new Property.ValueChangeListener() {
		@Override
		public void valueChange(Property.ValueChangeEvent valueChangeEvent) {
			final BulkAction bulkAction = (BulkAction) valueChangeEvent.getProperty().getValue();
			switch (bulkAction) {
				case EXPORT:
					LoggingExcelExport excelExport = new LoggingExcelExport(baseTable, i18n.get("DoctorsView.heading"), i18n.get("DoctorsView.heading"), "doctors.xls", false, excelExportLogService, i18n.get("DoctorsView.heading"));
					excelExport.excludeCollapsedColumns();
					Iterator iterator = excelExport.getPropIds().iterator();

					while (iterator.hasNext()) {
						Object propId = iterator.next();
						if ("remove".equals(propId)) {
							iterator.remove();
						}
					}
					excelExport.export();
					break;
			}

			otherComboBox.removeValueChangeListener(otherComboBoxValueChangeListener);
			otherComboBox.setValue(null);
			otherComboBox.addValueChangeListener(otherComboBoxValueChangeListener);
		}
	};

	private final Button.ClickListener filterCellClickListener = new Button.ClickListener() {
		@Override
		public void buttonClick(Button.ClickEvent clickEvent) {
			TableObject tableObject = (TableObject) clickEvent.getButton().getData();
			Doctor doctor = doctorService.findOne((Long) tableObject.getItemId());
			List<String> patientIds;
			PatientFilter patientFilter;

			switch ((String) tableObject.getColumnId()) {
				case "patients":
					patientIds = new ArrayList<>();
					patientFilter = new PatientFilter();
					for (Patient patient : doctor.getPatients()) {
						patientIds.add(FilterUtil.getFilterString(FilterOperand.EQUAL_TO, String.valueOf(patient.getId())));
					}
					patientFilter.setIds(patientIds);
					VaadinSession.getCurrent().setAttribute(PatientFilter.class, patientFilter);
					UI.getCurrent().getNavigator().navigateTo(PatientsView.NAME);
					break;
			}
		}
	};

	@Override
	protected void init() {
		super.defaultComponents(i18n.get("DoctorsView.heading"));
		initTable();
	}

	@Override
	protected void customizeView() {
		Label expandingGap = new Label();
		expandingGap.setWidth("100%");

		getHeaderToolBarHorizontalLayout().setWidth(100, Unit.PERCENTAGE);
		getHeaderToolBarHorizontalLayout().setSpacing(true);
		getHeaderToolBarHorizontalLayout().addComponent(getQuickSearchComponent());
		getHeaderToolBarHorizontalLayout().addComponent(getFilterComboBox());
		getHeaderToolBarHorizontalLayout().addComponent(expandingGap);
		getHeaderToolBarHorizontalLayout().addComponent(getOtherComboBox());
		getHeaderToolBarHorizontalLayout().setExpandRatio(expandingGap, 1);

		if (doctorService.userHasPermission(PermissionName.PERM_DOCTOR_CREATE_ANY)) {
			setAddButton(i18n.get("DoctorsView.addButton"));
		}
	}

	@Override
	protected void registerColumns() {
		baseTable.addStyleName("doctorsView-view");
		baseTable.setHeight(100, Unit.PERCENTAGE);
		baseTable.addStyleName(ValoTheme.TABLE_SMALL);
		baseTable.setColumnCollapsingAllowed(true);
		baseTable.setColumnReorderingAllowed(true);

		addGeneratedColumn("displayName", new EditCell(editClickListener));
		addGeneratedColumn("status", new StatusCell());
		addGeneratedColumn("patients", new CountFilterCell(filterCellClickListener));
		addGeneratedColumn("remove", new RemoveCell<>(UserService.class));

		addContainerProperty("displayName", String.class, null);
		addContainerProperty("userName", String.class, null);
		addContainerProperty("status", EntityStatus.class, EntityStatus.ACTIVE);
		addContainerProperty("patients", Integer.class, 0);
		addContainerProperty("isEditCellEnabled", Boolean.class, false);
		addContainerProperty("isRemoveCellEnabled", Boolean.class, false);
		addContainerProperty("isVisible", Boolean.class, false);

		baseTable.setColumnAlignment("patients", Table.Align.RIGHT);

		baseTable.setColumnHeader("displayName", i18n.get("UsersView.displayName"));
		baseTable.setColumnHeader("userName", i18n.get("UsersView.userName"));
		baseTable.setColumnHeader("status", i18n.get("UsersView.status"));
		baseTable.setColumnHeader("patients", i18n.get("UsersView.patients"));
		baseTable.setColumnHeader("remove", "");
	}

	@Override
	protected Doctor getFormEntity() {
		return new Doctor();
	}

	@Override
	protected void generateForm(Doctor tEntity) {
		form = new DoctorForm(tEntity, true);
	}

	@Override
	protected void setGridDataSource() {
		DoctorFilter sessionDoctorFilter = VaadinSession.getCurrent().getAttribute(DoctorFilter.class);
		if (sessionDoctorFilter != null) {
			serviceGroupFilter = sessionDoctorFilter;
			VaadinSession.getCurrent().setAttribute(DoctorFilter.class, null);
		}

		if (FilterUtil.isFilterActive(serviceGroupFilter)) {
			try {
				filterInfoVerticalLayout.removeComponent(filterInfo);
			} catch (Exception e) {
			}
			filterInfo = new FilterInfo<>(serviceGroupFilter);
			filterInfoVerticalLayout.addComponent(filterInfo, 0);
		} else {
			try {
				quickSearchTextField.setValue("");
				filterInfoVerticalLayout.removeComponent(filterInfo);
			} catch (Exception e) {
			}
		}

		List<Doctor> doctors = doctorService.findAllWithFilter(serviceGroupFilter);
		if (filterInfo != null) {
			filterInfo.setEntityCount(doctors.size());
		}

		baseTable.setContainerDataSource(null);
		for (Doctor doctor : doctors) {
			doctor.setDisplayName();
			Item item = indexedContainer.addItem(doctor.getId());
			item.getItemProperty("displayName").setValue(doctor.getDisplayName());
			item.getItemProperty("userName").setValue(doctor.getUserName());
			item.getItemProperty("status").setValue(doctor.getStatus());
			item.getItemProperty("patients").setValue(doctor.getPatients().size());
			item.getItemProperty("isVisible").setValue(Boolean.TRUE);
			if (doctorService.userHasPermission(PermissionName.PERM_DOCTOR_UPDATE_ANY)) {
				item.getItemProperty("isEditCellEnabled").setValue(Boolean.TRUE);
			}
			if (doctorService.userHasPermission(PermissionName.PERM_DOCTOR_DELETE_ANY)) {
				item.getItemProperty("isRemoveCellEnabled").setValue(Boolean.TRUE);
			}
		}
		baseTable.setContainerDataSource(indexedContainer);
		setVisibleColumns(visibleColumns.toArray());
	}

	@Override
	protected void setBaseTable() {
		baseTable = new BaseTable(null, indexedContainer);
		baseTable.setCacheRate(2);

		if (filterInfoVerticalLayout.getComponentCount() != 0) {
			filterInfoVerticalLayout = new VerticalLayout();
			serviceGroupFilter = new DoctorFilter();
		}

		filterInfoVerticalLayout.setImmediate(true);
		filterInfoVerticalLayout.setSizeFull();
		filterInfoVerticalLayout.addComponent(baseTable);
		filterInfoVerticalLayout.setExpandRatio(baseTable, 1);
		addComponent(filterInfoVerticalLayout);
		setExpandRatio(filterInfoVerticalLayout, 1);
	}

	@Override
	public void windowClose(Window.CloseEvent closeEvent) {

	}

	private Component getQuickSearchComponent() {
		quickSearchTextField.addStyleName(ValoTheme.TEXTFIELD_SMALL);
		quickSearchTextField.setInputPrompt(i18n.get("BaseForm.quickSearch"));
		quickSearchTextField.addValueChangeListener(quickSearchValueChangeListener);

		return quickSearchTextField;
	}

	private ComboBox getFilterComboBox() {
		filterComboBox.setWidth(90, Unit.PIXELS);
		filterComboBox.addStyleName(ValoTheme.COMBOBOX_SMALL);
		filterComboBox.setNullSelectionAllowed(false);
		filterComboBox.setTextInputAllowed(false);
		filterComboBox.setInputPrompt(i18n.get("BaseView.filterLabel"));
		setFilterComboBoxItems();
		filterComboBox.addValueChangeListener(filterComboBoxValueChangeListener);

		return filterComboBox;
	}

	private void setFilterComboBoxItems() {
		filterComboBox.removeAllItems();
		for (DoctorFilter doctorFilter : filterService.findAllByUser(UserUtil.getUser(), FilterCategory.DOCTOR, DoctorFilter.class)) {
			filterComboBox.addItem(doctorFilter);
			filterComboBox.setItemCaption(doctorFilter, doctorFilter.getFilterName());
		}

		filterComboBox.addItem("showFilter");
		filterComboBox.setItemCaption("showFilter", i18n.get("BaseView.showFilterLabel"));
	}

	private ComboBox getOtherComboBox() {
		otherComboBox.setWidth(90, Unit.PIXELS);
		otherComboBox.addStyleName(ValoTheme.COMBOBOX_SMALL);
		otherComboBox.setNullSelectionAllowed(false);
		otherComboBox.setTextInputAllowed(false);
		otherComboBox.setInputPrompt(i18n.get("GridView.otherLabel"));

		otherComboBox.addItem(BulkAction.EXPORT);
		otherComboBox.setItemCaption(BulkAction.EXPORT, i18n.get("GridView.otherExportLabel"));

		otherComboBox.addValueChangeListener(otherComboBoxValueChangeListener);

		return otherComboBox;
	}

	@EventBusListenerMethod
	public void buttonClick(EventBusAction eventBusAction) {
		if (eventBusAction.getFilter() != null && DoctorFilter.class.getName().equals(eventBusAction.getFilter().getClass().getName())) {
			if (eventBusAction.getAction().equals(Action.FILTER)) {
				serviceGroupFilter = (DoctorFilter) eventBusAction.getFilter();
				refreshTable();
			} else if (eventBusAction.getAction().equals(Action.DELETE_FILTER)) {
				quickSearchTextField.removeValueChangeListener(quickSearchValueChangeListener);
				serviceGroupFilter = (DoctorFilter) eventBusAction.getFilter();
				quickSearchTextField.setValue("");
				quickSearchTextField.addValueChangeListener(quickSearchValueChangeListener);
				refreshTable();
			} else if (eventBusAction.getAction().equals(Action.SAVE_FILTER)) {
				setFilterComboBoxItems();
			}
		}
	}
}
