package hu.oroscom.c1801.view;

import com.vaadin.data.Item;
import com.vaadin.data.Property;
import com.vaadin.server.VaadinSession;
import com.vaadin.ui.*;
import com.vaadin.ui.themes.ValoTheme;
import hu.oroscom.c1801.NeededAnyPermission;
import hu.oroscom.c1801.component.FilterInfo;
import hu.oroscom.c1801.component.cell.DateTimeFormatCell;
import hu.oroscom.c1801.component.cell.DownloadCSVCell;
import hu.oroscom.c1801.component.form.ExcelExportLogForm;
import hu.oroscom.c1801.component.table.BaseTable;
import hu.oroscom.c1801.component.windowed.FilterPanel;
import hu.oroscom.c1801.domain.ExcelExportLog;
import hu.oroscom.c1801.domain.filter.ExcelExportLogFilter;
import hu.oroscom.c1801.domain.filter.PatientFilter;
import hu.oroscom.c1801.enumeration.Action;
import hu.oroscom.c1801.enumeration.FilterCategory;
import hu.oroscom.c1801.enumeration.PermissionName;
import hu.oroscom.c1801.service.ExcelExportLogService;
import hu.oroscom.c1801.service.FilterService;
import hu.oroscom.c1801.util.EventBusAction;
import hu.oroscom.c1801.util.FilterUtil;
import hu.oroscom.c1801.util.UserUtil;
import org.joda.time.DateTime;
import org.vaadin.spring.annotation.VaadinUIScope;
import org.vaadin.spring.events.annotation.EventBusListenerMethod;
import org.vaadin.spring.navigator.annotation.VaadinView;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Created by kollerk on 2014.10.07..
 */
@VaadinUIScope
@VaadinView(name = ExcelExportLogsView.NAME)
@NeededAnyPermission(values = {
		PermissionName.PERM_ROLE_CREATE_ANY
})
public class ExcelExportLogsView extends GridView<ExcelExportLog, ExcelExportLogService, ExcelExportLogForm> {
	public static final String NAME = "excelExportLogs";
	private final List<Object> visibleColumns = new ArrayList<>(Arrays.asList(new Object[]{
			"downloadData",
			"viewName",
			"exportDate",
			"exportedUserName",
			"exportedRowCount",
	}));
	private ComboBox filterComboBox = new ComboBox();
	private TextField quickSearchTextField = new TextField();
	private ExcelExportLogFilter excelExportLogFilter = new ExcelExportLogFilter();
	private VerticalLayout filterInfoVerticalLayout = new VerticalLayout();
	private FilterInfo filterInfo;

	@Inject
	ExcelExportLogService excelExportLogService;

	@Inject
	FilterService filterService;

	private final Property.ValueChangeListener filterComboBoxValueChangeListener = new Property.ValueChangeListener() {
		@Override
		public void valueChange(Property.ValueChangeEvent valueChangeEvent) {
			if ("showFilter".equals(valueChangeEvent.getProperty().getValue())) {
				FilterPanel filterPanel = new FilterPanel(excelExportLogFilter, filterService, excelExportLogService);
				addContentToWindow(filterPanel);
			} else {
				excelExportLogFilter = (ExcelExportLogFilter) valueChangeEvent.getProperty().getValue();
				refreshTable();
			}

			filterComboBox.removeValueChangeListener(filterComboBoxValueChangeListener);
			filterComboBox.setValue(null);
			filterComboBox.addValueChangeListener(filterComboBoxValueChangeListener);
		}
	};

	private final Property.ValueChangeListener quickSearchValueChangeListener = new Property.ValueChangeListener() {
		@Override
		public void valueChange(Property.ValueChangeEvent event) {
			String searchText = event.getProperty().getValue().toString();
			if (searchText.length() > 2) {
				excelExportLogFilter.setQuickSearch(searchText);
				eventBus.publish(this, new EventBusAction<>(Action.FILTER, excelExportLogFilter));
			} else {
				Notification.show(i18n.get("QuickSearch.minThreeCharacter"), Notification.Type.WARNING_MESSAGE);
			}
		}
	};

	@Override
	protected void init() {
		super.defaultComponents(i18n.get("ExcelExportLogsView.exportedExcelsLabel"));
		initTable();
	}

	@Override
	protected void customizeView() {
		Label expandingGap = new Label();
		expandingGap.setWidth("100%");

		getHeaderToolBarHorizontalLayout().setWidth(100, Unit.PERCENTAGE);
		getHeaderToolBarHorizontalLayout().setSpacing(true);
		getHeaderToolBarHorizontalLayout().addComponent(getQuickSearchComponent());
		getHeaderToolBarHorizontalLayout().addComponent(getFilterComboBox());
		getHeaderToolBarHorizontalLayout().addComponent(expandingGap);
		getHeaderToolBarHorizontalLayout().setExpandRatio(expandingGap, 1);

	}

	@Override
	protected void registerColumns() {
		baseTable.addStyleName("excelExportLogs-view");
		baseTable.setHeight(100, Unit.PERCENTAGE);
		baseTable.addStyleName(ValoTheme.TABLE_SMALL);
		baseTable.setColumnCollapsingAllowed(true);
		baseTable.setColumnReorderingAllowed(true);

		addGeneratedColumn("downloadData", new DownloadCSVCell());
		addGeneratedColumn("exportDate", new DateTimeFormatCell());

		addContainerProperty("downloadData", String.class, "");
		addContainerProperty("exportedUserName", String.class, "");
		addContainerProperty("viewName", String.class, "");
		addContainerProperty("exportDate", DateTime.class, null);
		addContainerProperty("exportedRowCount", Integer.class, 0);

		addContainerProperty("isEditCellEnabled", Boolean.class, false);

		baseTable.setColumnHeader("downloadData", i18n.get("ExcelExportLogsView.downloadData"));
		baseTable.setColumnHeader("viewName", i18n.get("ExcelExportLogsView.viewName"));
		baseTable.setColumnHeader("exportDate", i18n.get("ExcelExportLogsView.exportDate"));
		baseTable.setColumnHeader("exportedUserName", i18n.get("ExcelExportLogsView.exportedUserName"));
		baseTable.setColumnHeader("exportedRowCount", i18n.get("ExcelExportLogsView.exportedRowCount"));
	}

	@Override
	protected ExcelExportLog getFormEntity() {
		return new ExcelExportLog();
	}

	@Override
	protected void generateForm(ExcelExportLog tEntity) {
		form = new ExcelExportLogForm(tEntity, true);
	}

	@Override
	protected void setGridDataSource() {
		ExcelExportLogFilter sessionExcelExportLogFilter = VaadinSession.getCurrent().getAttribute(ExcelExportLogFilter.class);
		if (sessionExcelExportLogFilter != null) {
			excelExportLogFilter = sessionExcelExportLogFilter;
			VaadinSession.getCurrent().setAttribute(PatientFilter.class, null);
		}

		if (FilterUtil.isFilterActive(excelExportLogFilter)) {
			try {
				filterInfoVerticalLayout.removeComponent(filterInfo);
			} catch (Exception e) {
			}
			filterInfo = new FilterInfo<>(excelExportLogFilter);
			filterInfoVerticalLayout.addComponent(filterInfo, 0);
		} else {
			try {
				quickSearchTextField.setValue("");
				filterInfoVerticalLayout.removeComponent(filterInfo);
			} catch (Exception e) {
			}
		}

		List<ExcelExportLog> excelExportLogs = excelExportLogService.findAllWithFilter(excelExportLogFilter);
		if (filterInfo != null) {
			filterInfo.setEntityCount(excelExportLogs.size());
		}

		baseTable.setContainerDataSource(null);
		for (ExcelExportLog excelExportLog : excelExportLogs) {
			Item item = indexedContainer.addItem(excelExportLog.getId());
			item.getItemProperty("downloadData").setValue(excelExportLog.getExportedCsv());
			item.getItemProperty("viewName").setValue(excelExportLog.getViewName());
			item.getItemProperty("exportDate").setValue(excelExportLog.getExportDate());
			item.getItemProperty("exportedUserName").setValue(excelExportLog.getExportedUserName());
			item.getItemProperty("exportedRowCount").setValue(excelExportLog.getExportedRowCount());

			item.getItemProperty("isEditCellEnabled").setValue(Boolean.TRUE);
		}
		baseTable.setContainerDataSource(indexedContainer);
		setVisibleColumns(visibleColumns.toArray());
	}

	@Override
	protected void setBaseTable() {
		baseTable = new BaseTable(null, indexedContainer);
		baseTable.setCacheRate(2);

		if (filterInfoVerticalLayout.getComponentCount() != 0) {
			filterInfoVerticalLayout = new VerticalLayout();
			excelExportLogFilter = new ExcelExportLogFilter();
		}

		filterInfoVerticalLayout.setImmediate(true);
		filterInfoVerticalLayout.setSizeFull();
		filterInfoVerticalLayout.addComponent(baseTable);
		filterInfoVerticalLayout.setExpandRatio(baseTable, 1);
		addComponent(filterInfoVerticalLayout);
		setExpandRatio(filterInfoVerticalLayout, 1);
	}

	@Override
	public void windowClose(Window.CloseEvent closeEvent) {

	}

	private Component getQuickSearchComponent() {
		quickSearchTextField.addStyleName(ValoTheme.TEXTFIELD_SMALL);
		quickSearchTextField.setInputPrompt(i18n.get("BaseForm.quickSearch"));
		quickSearchTextField.addValueChangeListener(quickSearchValueChangeListener);

		return quickSearchTextField;
	}

	private ComboBox getFilterComboBox() {
		filterComboBox.setWidth(90, Unit.PIXELS);
		filterComboBox.addStyleName(ValoTheme.COMBOBOX_SMALL);
		filterComboBox.setNullSelectionAllowed(false);
		filterComboBox.setTextInputAllowed(false);
		filterComboBox.setInputPrompt(i18n.get("BaseView.filterLabel"));
		setFilterComboBoxItems();
		filterComboBox.addValueChangeListener(filterComboBoxValueChangeListener);

		return filterComboBox;
	}

	private void setFilterComboBoxItems() {
		filterComboBox.removeAllItems();
		for (ExcelExportLogFilter excelExportLogFilter : filterService.findAllByUser(UserUtil.getUser(), FilterCategory.EXPORTED_EXCELS, ExcelExportLogFilter.class)) {
			filterComboBox.addItem(excelExportLogFilter);
			filterComboBox.setItemCaption(excelExportLogFilter, excelExportLogFilter.getFilterName());
		}

		filterComboBox.addItem("showFilter");
		filterComboBox.setItemCaption("showFilter", i18n.get("BaseView.showFilterLabel"));
	}

	@EventBusListenerMethod
	public void buttonClick(EventBusAction eventBusAction) {
		if (eventBusAction.getFilter() != null && ExcelExportLogFilter.class.getName().equals(eventBusAction.getFilter().getClass().getName())) {
			if (eventBusAction.getAction().equals(Action.FILTER)) {
				excelExportLogFilter = (ExcelExportLogFilter) eventBusAction.getFilter();
				refreshTable();
			} else if (eventBusAction.getAction().equals(Action.DELETE_FILTER)) {
				excelExportLogFilter = (ExcelExportLogFilter) eventBusAction.getFilter();
				quickSearchTextField.setValue("");
				refreshTable();
			} else if (eventBusAction.getAction().equals(Action.SAVE_FILTER)) {
				setFilterComboBoxItems();
			}
		} else if (eventBusAction.getEntity() != null && ExcelExportLog.class.getName().equals(eventBusAction.getEntity().getClass().getName())) {
			if (eventBusAction.getAction().equals(Action.EDIT)) {
				editViewAndAddFormToWindow((ExcelExportLog) eventBusAction.getEntity());
			}
		}
	}
}
