package hu.oroscom.c1801.view;

import com.vaadin.data.util.IndexedContainer;
import com.vaadin.navigator.View;
import com.vaadin.navigator.ViewChangeListener;
import com.vaadin.server.VaadinSession;
import com.vaadin.ui.Button;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.Table;
import com.vaadin.ui.UI;
import com.vaadin.ui.themes.ValoTheme;
import hu.oroscom.c1801.component.form.BaseForm;
import hu.oroscom.c1801.component.table.BaseTable;
import hu.oroscom.c1801.domain.BaseEntity;
import hu.oroscom.c1801.domain.GridSetting;
import hu.oroscom.c1801.domain.User;
import hu.oroscom.c1801.enumeration.Action;
import hu.oroscom.c1801.listener.ColumnCollapsedEvent;
import hu.oroscom.c1801.listener.ColumnSortEvent;
import hu.oroscom.c1801.service.BaseService;
import hu.oroscom.c1801.service.GridSettingService;
import hu.oroscom.c1801.util.EventBusAction;
import org.vaadin.spring.events.annotation.EventBusListenerMethod;

import javax.inject.Inject;
import java.util.*;

/**
 * Created by Norbert on 2014.11.24..
 */
public abstract class GridView<T extends BaseEntity, S extends BaseService, F extends BaseForm<T, S>> extends BaseView {

	protected final Button.ClickListener editClickListener = new Button.ClickListener() {
		@Override
		public void buttonClick(Button.ClickEvent clickEvent) {
			Long id = (Long) clickEvent.getButton().getData();
			editViewAndAddFormToWindow(id);
		}
	};

	private final Button.ClickListener addClickListener = new Button.ClickListener() {
		@Override
		public void buttonClick(Button.ClickEvent clickEvent) {
			addButtonClickEvent(clickEvent);
			editViewAndAddFormToWindow(getFormEntity());
		}
	};

	protected Set<View> loadedViews = new HashSet<>();

	@Inject
	protected S service;

	@Inject
	GridSettingService gridSettingService;

	protected IndexedContainer indexedContainer;
	protected BaseTable baseTable;
	protected F form;
	protected boolean viewChanged = false;
	protected GridSetting gridSetting;
	private final Set<Object> selectedItemIds = new HashSet<>();
	private String navigationState;
	private Button addButton;

	private final Button.ClickListener backClickListener = new Button.ClickListener() {
		@Override
		public void buttonClick(Button.ClickEvent clickEvent) {
			UI.getCurrent().getNavigator().navigateTo(navigationState);
		}
	};
	private final ColumnCollapsedEvent.ColumnCollapsedListener columnCollapsedListener = new ColumnCollapsedEvent.ColumnCollapsedListener() {
		@Override
		public void columnCollapsed(ColumnCollapsedEvent columnCollapsedEvent) {
			gridSetting.setCollapsedColumns(baseTable.getCollapsedColumns());
			gridSetting = gridSettingService.save(gridSetting);
		}
	};
	private final Table.ColumnReorderListener columnReorderListener = new Table.ColumnReorderListener() {
		@Override
		public void columnReorder(Table.ColumnReorderEvent columnReorderEvent) {
			gridSetting.setVisibleColumns(Arrays.asList(((BaseTable) columnReorderEvent.getSource()).getVisibleColumns()));
			gridSetting = gridSettingService.save(gridSetting);
		}
	};
	private final ColumnSortEvent.ColumnSortListener columnSortListener = new ColumnSortEvent.ColumnSortListener() {
		@Override
		public void columnSort(ColumnSortEvent columnSortEvent) {
			gridSetting.setSortColumn((String) columnSortEvent.getPropertyId());
			gridSetting.setSortAscending(columnSortEvent.isSortAscending());
			gridSetting = gridSettingService.save(gridSetting);
		}
	};

	private HorizontalLayout headerToolBarHorizontalLayout;

	protected void initTable() {
		headerToolBarHorizontalLayout = new HorizontalLayout();
		addComponent(headerToolBarHorizontalLayout);
		customizeView();
		setBeanContainer();
		setBaseTable();
		registerColumns();
	}

	protected abstract void customizeView();

	protected abstract void registerColumns();

	protected abstract T getFormEntity();

	protected abstract void generateForm(T tEntity);

	protected abstract void setGridDataSource();

	protected void addGeneratedColumn(Object id, Table.ColumnGenerator columnGenerator) {
		if ("remove".equals(id)) {
			baseTable.setColumnWidth("remove", 40);
			baseTable.setColumnAlignment("remove", Table.Align.CENTER);
		}
		baseTable.addGeneratedColumn(id, columnGenerator);
	}

	protected void setVisibleColumns(Object... columns) {
		baseTable.setVisibleColumns(columns);
	}

	protected void setBackButton(String navigationState) {
		this.navigationState = navigationState;
		Button backButton = new Button("<", backClickListener);
		backButton.addStyleName(ValoTheme.BUTTON_SMALL);
		headerToolBarHorizontalLayout.addComponent(backButton);
	}

	protected void setAddButton(String caption) {
		addButton = new Button(caption, addClickListener);
		addButton.addStyleName("view-heading");
		addButton.addStyleName(ValoTheme.BUTTON_SMALL);
		headerTitleHorizontalLayout.addComponent(addButton);
	}

	protected void setBeanContainer() {
		indexedContainer = new IndexedContainer();
	}

	protected void setBaseTable() {
		baseTable = new BaseTable(null, indexedContainer);
		baseTable.setCacheRate(2);
		addComponent(baseTable);
		setExpandRatio(baseTable, 1);
	}

	protected void refreshTable() {
		indexedContainer.removeAllItems();
		setGridDataSource();
		baseTable.setPageLength(50);
		baseTable.refreshRowCache();
		viewChanged = false;
		getSelectedItemIds().clear();
		applyGridSetting();
	}

	protected void addContainerProperty(Object id, Class<?> type, Object defaultValue) {
		indexedContainer.addContainerProperty(id, type, defaultValue);
	}

	protected void addButtonClickEvent(Button.ClickEvent clickEvent) {
	}

	private void editViewAndAddFormToWindow(Long id) {
		generateForm((T) service.findOne(id));
		addFormToWindow(form);
	}

	protected void editViewAndAddFormToWindow(T tEntity) {
		if (tEntity.getId() != null) {
			tEntity = (T) service.findOne(tEntity.getId());
		}
		generateForm(tEntity);
		addFormToWindow(form);
	}

	@Override
	public void enter(ViewChangeListener.ViewChangeEvent viewChangeEvent) {
//		super.enter(viewChangeEvent);
		loadedViews.add(viewChangeEvent.getNewView());
		viewChanged = true;
		if (viewChangeEvent.getOldView() != viewChangeEvent.getNewView()) {
			refreshTable();
//			applyGridSetting();
		}
	}

	private void applyGridSetting() {
		if (baseTable == null || !baseTable.isColumnCollapsingAllowed() || !baseTable.isColumnReorderingAllowed()) {
			return;
		}
		baseTable.removeColumnCollapsedListener(columnCollapsedListener);
		baseTable.removeColumnReorderListener(columnReorderListener);
		baseTable.removeColumnSortListener(columnSortListener);
		try {
			java.lang.reflect.Field nameField = this.getClass().getDeclaredField("NAME");
			nameField.setAccessible(true);
			Object value = nameField.get(this);
			if (value != null) {
				gridSetting = gridSettingService.findGridSettingByGridName(value.toString());

				if (gridSetting == null) {
					gridSetting = new GridSetting();
					gridSetting.setUser(VaadinSession.getCurrent().getAttribute(User.class));
					gridSetting.setGridName(value.toString());
					gridSetting.setCollapsedColumns(baseTable.getCollapsedColumns());
					gridSetting.setVisibleColumns(Arrays.asList(baseTable.getVisibleColumns()));
					gridSetting.setSortColumn((String) baseTable.getSortContainerPropertyId());
					gridSetting.setSortAscending(baseTable.isSortAscending());
				} else {
					if (gridSetting.getVisibleColumns() != null) {
						List<Object> defaultVisibleColumns;
						List<Object> oldVisibleColumns;
						if (baseTable.getVisibleColumns().length > gridSetting.getVisibleColumns().size()) {
							defaultVisibleColumns = new ArrayList<>(Arrays.asList(baseTable.getVisibleColumns()));
							oldVisibleColumns = new ArrayList<>(gridSetting.getVisibleColumns());
							defaultVisibleColumns.removeAll(oldVisibleColumns);
							oldVisibleColumns.addAll(defaultVisibleColumns);
							gridSetting.setVisibleColumns(oldVisibleColumns);
						}
						if (baseTable.getVisibleColumns().length < gridSetting.getVisibleColumns().size()) {
							defaultVisibleColumns = new ArrayList<>(Arrays.asList(baseTable.getVisibleColumns()));
							oldVisibleColumns = new ArrayList<>(gridSetting.getVisibleColumns());
							List<Object> newVisibleColumns = new ArrayList<>(gridSetting.getVisibleColumns());
							oldVisibleColumns.removeAll(defaultVisibleColumns);
							newVisibleColumns.removeAll(oldVisibleColumns);
							gridSetting.setVisibleColumns(newVisibleColumns);
						}
						baseTable.setVisibleColumns(gridSetting.getVisibleColumns().toArray());
					}
					if (gridSetting.getCollapsedColumns() != null) {
						Set<Object> collapsedColumns = new HashSet<>(baseTable.getCollapsedColumns());
						for (Object column : collapsedColumns) {
							baseTable.setColumnCollapsed(column, false);
						}
						for (Object column : gridSetting.getCollapsedColumns()) {
							baseTable.setColumnCollapsed(column, true);
						}
					}
					if (gridSetting.getSortColumn() != null) {
						baseTable.setSortContainerPropertyId(gridSetting.getSortColumn());
						baseTable.setSortAscending(gridSetting.isSortAscending());
						indexedContainer.sort(new Object[]{gridSetting.getSortColumn()}, new boolean[]{gridSetting.isSortAscending()});
					}
				}
			}
		} catch (NoSuchFieldException | IllegalAccessException e) {
			e.printStackTrace();
		}

		baseTable.addColumnCollapsedListener(columnCollapsedListener);
		baseTable.addColumnReorderListener(columnReorderListener);
		baseTable.addColumnSortListener(columnSortListener);
	}

	@EventBusListenerMethod
	private void saveButtonClick(EventBusAction eventBusAction) {
		if (eventBusAction.getAction() == Action.SAVE || eventBusAction.getAction() == Action.DELETE) {
			refreshTable();
		}

		super.buttonClicked(eventBusAction);
	}

	public HorizontalLayout getHeaderToolBarHorizontalLayout() {
		return headerToolBarHorizontalLayout;
	}

	public Button.ClickListener getAddClickListener() {
		return addClickListener;
	}

	public Button getAddButton() {
		return addButton;
	}

	public Set<Object> getSelectedItemIds() {
		return selectedItemIds;
	}
}
