package hu.oroscom.c1801.view;

import com.vaadin.data.Item;
import com.vaadin.data.Property;
import com.vaadin.data.util.BeanItemContainer;
import com.vaadin.navigator.Navigator;
import com.vaadin.server.FontAwesome;
import com.vaadin.server.VaadinSession;
import com.vaadin.ui.*;
import com.vaadin.ui.themes.ValoTheme;
import hu.oroscom.c1801.NeededAnyPermission;
import hu.oroscom.c1801.component.FilterInfo;
import hu.oroscom.c1801.component.cell.*;
import hu.oroscom.c1801.component.form.PatientAssignmentPanel;
import hu.oroscom.c1801.component.form.PatientForm;
import hu.oroscom.c1801.component.form.ServiceGroupAssignmentPanel;
import hu.oroscom.c1801.component.form.UserCommentForm;
import hu.oroscom.c1801.component.table.BaseTable;
import hu.oroscom.c1801.component.windowed.FilterPanel;
import hu.oroscom.c1801.domain.Patient;
import hu.oroscom.c1801.domain.ServiceGroup;
import hu.oroscom.c1801.domain.UserComment;
import hu.oroscom.c1801.domain.filter.PatientFilter;
import hu.oroscom.c1801.enumeration.*;
import hu.oroscom.c1801.service.*;
import hu.oroscom.c1801.util.EventBusAction;
import hu.oroscom.c1801.util.FilterUtil;
import hu.oroscom.c1801.util.LoggingExcelExport;
import hu.oroscom.c1801.util.UserUtil;
import org.apache.commons.lang.StringUtils;
import org.vaadin.spring.annotation.VaadinUIScope;
import org.vaadin.spring.events.annotation.EventBusListenerMethod;
import org.vaadin.spring.navigator.annotation.VaadinView;

import javax.inject.Inject;
import java.util.*;

@VaadinUIScope
@VaadinView(name = PatientsView.NAME)
@NeededAnyPermission(values = {
		PermissionName.PERM_PATIENT_SELECT_ANY
})
public class PatientsView extends GridView<Patient, PatientService, PatientForm> {
	public static final String NAME = "patients";
	private final List<Object> visibleColumns = new ArrayList<>(Arrays.asList(new Object[]{"multiSelect", "patientId", "userName", "patientStatus", "doctor", "studyEntries", "eatenFoods", "intolerantMaterials", "complaints", "remove"}));

	private Button selectAllButton = new Button();
	private ComboBox bulkActionComboBox = new ComboBox();
	private ComboBox otherComboBox = new ComboBox();
	private ComboBox filterComboBox = new ComboBox();
	private TextField quickSearchTextField = new TextField();
	private PatientFilter patientFilter = new PatientFilter();
	private VerticalLayout filterInfoVerticalLayout = new VerticalLayout();
	private FilterInfo filterInfo;

	@Inject
	private PatientService patientService;

	@Inject
	private ServiceGroupService serviceGroupService;

	@Inject
	private DoctorService doctorService;

	@Inject
	private FilterService filterService;

	@Inject
	private ExcelExportLogService excelExportLogService;

	@Inject
	private UserService userService;

	private final Property.ValueChangeListener filterComboBoxValueChangeListener = new Property.ValueChangeListener() {
		@Override
		public void valueChange(Property.ValueChangeEvent valueChangeEvent) {
			List<GlobalFilter> globalFilters = Arrays.asList(GlobalFilter.values());
			if ("showFilter".equals(valueChangeEvent.getProperty().getValue())) {
				FilterPanel filterPanel = new FilterPanel(patientFilter, filterService, patientService, doctorService, userService);
				addContentToWindow(filterPanel);
			} else if (globalFilters.contains(valueChangeEvent.getProperty().getValue())) {
				GlobalFilter globalFilter = (GlobalFilter) valueChangeEvent.getProperty().getValue();
				String filterName = "";
				patientFilter = new PatientFilter();
				patientFilter.setFilterType(FilterType.GLOBAL);

				switch (globalFilter) {
					case OWN:
						filterName = GlobalFilter.OWN.getCaption();
						List<String> doctorFilterStrings = new ArrayList<>();
						doctorFilterStrings.add(FilterUtil.getFilterString(FilterOperand.EQUAL_TO, UserUtil.getUser().getId().toString()));
						patientFilter.setDoctor(doctorFilterStrings);
						break;
				}

				patientFilter.setFilterName(filterName);
				refreshTable();
			} else {
				patientFilter = (PatientFilter) valueChangeEvent.getProperty().getValue();
				refreshTable();
			}

			filterComboBox.removeValueChangeListener(filterComboBoxValueChangeListener);
			filterComboBox.setValue(null);
			filterComboBox.addValueChangeListener(filterComboBoxValueChangeListener);
		}
	};

	private final Property.ValueChangeListener quickSearchValueChangeListener = new Property.ValueChangeListener() {
		@Override
		public void valueChange(Property.ValueChangeEvent event) {
			String searchText = event.getProperty().getValue().toString();
			if (searchText.length() > 2) {
				patientFilter.setQuickSearch(searchText);
				eventBus.publish(this, new EventBusAction<>(Action.FILTER, patientFilter));
			} else {
				Notification.show(i18n.get("QuickSearch.minThreeCharacter"), Notification.Type.WARNING_MESSAGE);
			}
		}
	};

	private final Property.ValueChangeListener otherComboBoxValueChangeListener = new Property.ValueChangeListener() {
		@Override
		public void valueChange(Property.ValueChangeEvent valueChangeEvent) {
			final BulkAction bulkAction = (BulkAction) valueChangeEvent.getProperty().getValue();
			switch (bulkAction) {
				case EXPORT:
					LoggingExcelExport excelExport = new LoggingExcelExport(baseTable, i18n.get("PatientsView.heading"), i18n.get("PatientsView.heading"), "patients.xls", false, excelExportLogService, i18n.get("PatientsView.heading"));
					excelExport.excludeCollapsedColumns();
					Iterator iterator = excelExport.getPropIds().iterator();

					while (iterator.hasNext()) {
						Object propId = iterator.next();
						if ("remove".equals(propId)) {
							iterator.remove();
						}
					}
					excelExport.export();
					break;
			}

			otherComboBox.removeValueChangeListener(otherComboBoxValueChangeListener);
			otherComboBox.setValue(null);
			otherComboBox.addValueChangeListener(otherComboBoxValueChangeListener);
		}
	};

	private Button.ClickListener selectAllButtonClickListener = new Button.ClickListener() {
		@Override
		public void buttonClick(Button.ClickEvent event) {
			boolean isSelectedItemIdsEmpty = getSelectedItemIds().isEmpty();
			if (isSelectedItemIdsEmpty) {
				getSelectedItemIds().addAll(indexedContainer.getItemIds());
			} else {
				getSelectedItemIds().clear();
			}
			for (Object itemId : indexedContainer.getItemIds()) {
				indexedContainer.getItem(itemId).getItemProperty("multiSelect").setValue(isSelectedItemIdsEmpty);
			}
			baseTable.refreshRowCache();
		}
	};

	private final Property.ValueChangeListener bulkActionComboBoxValueChangeListener = new Property.ValueChangeListener() {
		@Override
		public void valueChange(Property.ValueChangeEvent valueChangeEvent) {
			if (getSelectedItemIds().isEmpty() || valueChangeEvent.getProperty().getValue() == null) {
				bulkActionComboBox.setValue(null);
				Notification.show(i18n.get("GridView.noSelectedEntity"), Notification.Type.HUMANIZED_MESSAGE);
				return;
			}

			final Set<Object> selectedItemIds = getSelectedItemIds();

			if (!selectedItemIds.isEmpty()) {
				final BulkAction bulkAction = (BulkAction) valueChangeEvent.getProperty().getValue();
				if (BulkAction.MESSAGE_BROADCAST.equals(bulkAction)) {
					List<Patient> broadcastPatients = new ArrayList<>(patientService.findAll(selectedItemIds.toArray(new Long[selectedItemIds.size()])));

					UserComment userComment = new UserComment();
					userComment.setCustomer(UserUtil.getCustomer());
					userComment.setUserCommentType(UserCommentType.BROADCAST);
					userComment.setPatients(broadcastPatients);

					UserCommentForm form = new UserCommentForm(userComment, true);
					addFormToWindow(form);
				} else {
					addContentToWindow(generateAssignmentPanel(bulkAction));
				}
			}

			bulkActionComboBox.removeValueChangeListener(bulkActionComboBoxValueChangeListener);
			bulkActionComboBox.setValue(null);
			bulkActionComboBox.addValueChangeListener(bulkActionComboBoxValueChangeListener);
		}
	};

	private Button.ClickListener studyEntriesClickListener = new Button.ClickListener() {
		@Override
		public void buttonClick(Button.ClickEvent event) {
			Patient patient = patientService.findOne((Long) event.getButton().getData());
			Navigator navigator = UI.getCurrent().getNavigator();
			VaadinSession.getCurrent().setAttribute(Patient.class, patient);
			navigator.navigateTo(StudyEntriesView.NAME);
		}
	};

	private Button.ClickListener eatenFoodsClickListener = new Button.ClickListener() {
		@Override
		public void buttonClick(Button.ClickEvent event) {
			Patient patient = patientService.findOne((Long) event.getButton().getData());
			Navigator navigator = UI.getCurrent().getNavigator();
			VaadinSession.getCurrent().setAttribute(Patient.class, patient);
			navigator.navigateTo(EatenFoodsView.NAME);
		}
	};

	private Button.ClickListener intolerantMaterialsClickListener = new Button.ClickListener() {
		@Override
		public void buttonClick(Button.ClickEvent event) {
			Patient patient = patientService.findOne((Long) event.getButton().getData());
			Navigator navigator = UI.getCurrent().getNavigator();
			VaadinSession.getCurrent().setAttribute(Patient.class, patient);
			navigator.navigateTo(IntolerantMaterialsView.NAME);
		}
	};

	private Button.ClickListener complaintsClickListener = new Button.ClickListener() {
		@Override
		public void buttonClick(Button.ClickEvent event) {
			Patient patient = patientService.findOne((Long) event.getButton().getData());
			Navigator navigator = UI.getCurrent().getNavigator();
			VaadinSession.getCurrent().setAttribute(Patient.class, patient);
			navigator.navigateTo(ComplaintsView.NAME);
		}
	};

	private Component generateAssignmentPanel(BulkAction bulkAction) {
		switch (bulkAction) {
			case DOCTOR:
				return new PatientAssignmentPanel(getSelectedItemIds(), patientService, doctorService);
			case SERVICE_GROUP:
				return new ServiceGroupAssignmentPanel(getSelectedItemIds(), patientService, serviceGroupService);
		}

		return null;
	}

	@Override
	protected void init() {
		super.defaultComponents(i18n.get("PatientsView.heading"));
		initTable();
	}

	@Override
	protected void customizeView() {
		Label expandingGap = new Label();
		expandingGap.setWidth("100%");

		getHeaderToolBarHorizontalLayout().setWidth(100, Unit.PERCENTAGE);
		getHeaderToolBarHorizontalLayout().setSpacing(true);
		getHeaderToolBarHorizontalLayout().addComponent(getSelectAllButton());
		getHeaderToolBarHorizontalLayout().addComponent(getBulkActionComboBox());
		getHeaderToolBarHorizontalLayout().addComponent(getQuickSearchComponent());
		getHeaderToolBarHorizontalLayout().addComponent(getFilterComboBox());
		getHeaderToolBarHorizontalLayout().addComponent(expandingGap);
		getHeaderToolBarHorizontalLayout().addComponent(getOtherComboBox());
		getHeaderToolBarHorizontalLayout().setExpandRatio(expandingGap, 1);

		if (patientService.userHasPermission(PermissionName.PERM_PATIENT_CREATE_ANY)) {
			setAddButton(i18n.get("PatientsView.addButton"));
		}
	}

	@Override
	protected void registerColumns() {
		baseTable.addStyleName("patientsView-view");
		baseTable.setHeight(100, Unit.PERCENTAGE);
		baseTable.addStyleName(ValoTheme.TABLE_SMALL);
		baseTable.setColumnCollapsingAllowed(true);
		baseTable.setColumnReorderingAllowed(true);

		addGeneratedColumn("multiSelect", new MultiSelectCell(getSelectedItemIds()));
		addGeneratedColumn("patientId", new EditCell(editClickListener));
		addGeneratedColumn("patientStatus", new PatientStatusCell());
		addGeneratedColumn("studyEntries", new EditLinkCell(i18n.get("PatientsView.studyEntries"), studyEntriesClickListener));
		addGeneratedColumn("eatenFoods", new EditLinkCell(i18n.get("PatientsView.eatenFoods"), eatenFoodsClickListener));
		addGeneratedColumn("intolerantMaterials", new EditLinkCell(i18n.get("PatientsView.intolerantMaterials"), intolerantMaterialsClickListener));
		addGeneratedColumn("complaints", new EditLinkCell(i18n.get("PatientsView.complaints"), complaintsClickListener));
		addGeneratedColumn("remove", new RemoveCell<>(UserService.class));

		addContainerProperty("multiSelect", Boolean.class, false);
		addContainerProperty("patientId", String.class, null);
		addContainerProperty("userName", String.class, null);
		addContainerProperty("doctor", String.class, null);
		addContainerProperty("patientStatus", PatientStatus.class, PatientStatus.ACTIVE);
		addContainerProperty("isEditCellEnabled", Boolean.class, false);
		addContainerProperty("isRemoveCellEnabled", Boolean.class, false);
		addContainerProperty("isVisible", Boolean.class, false);

		baseTable.setColumnHeader("multiSelect", "");
		baseTable.setColumnHeader("patientId", i18n.get("PatientForm.patientId"));
		baseTable.setColumnHeader("userName", i18n.get("UsersView.userName"));
		baseTable.setColumnHeader("doctor", i18n.get("PatientsView.doctor"));
		baseTable.setColumnHeader("patientStatus", i18n.get("PatientForm.patientStatus"));
		baseTable.setColumnHeader("studyEntries", i18n.get("PatientsView.studyEntries"));
		baseTable.setColumnHeader("eatenFoods", i18n.get("PatientsView.eatenFoods"));
		baseTable.setColumnHeader("intolerantMaterials", i18n.get("PatientsView.intolerantMaterials"));
		baseTable.setColumnHeader("complaints", i18n.get("PatientsView.complaints"));
		baseTable.setColumnHeader("remove", "");

		baseTable.setColumnWidth("multiSelect", 28);
	}

	@Override
	protected Patient getFormEntity() {
		return new Patient();
	}

	@Override
	protected void generateForm(Patient tEntity) {
		BeanItemContainer<ServiceGroup> serviceGroupBeanItemContainer = new BeanItemContainer<>(ServiceGroup.class, serviceGroupService.findAllByCustomer());
		form = new PatientForm(tEntity, true, serviceGroupBeanItemContainer);
	}

	@Override
	protected void setGridDataSource() {
		PatientFilter sessionPatientFilter = VaadinSession.getCurrent().getAttribute(PatientFilter.class);
		if (sessionPatientFilter != null) {
			patientFilter = sessionPatientFilter;
			VaadinSession.getCurrent().setAttribute(PatientFilter.class, null);
		}

		if (FilterUtil.isFilterActive(patientFilter)) {
			try {
				filterInfoVerticalLayout.removeComponent(filterInfo);
			} catch (Exception e) {
			}
			filterInfo = new FilterInfo<>(patientFilter, doctorService);
			filterInfoVerticalLayout.addComponent(filterInfo, 0);
		} else {
			try {
				quickSearchTextField.setValue("");
				filterInfoVerticalLayout.removeComponent(filterInfo);
			} catch (Exception e) {
			}
		}

		List<Patient> patients = patientService.findAllWithFilter(patientFilter);
		if (filterInfo != null) {
			filterInfo.setEntityCount(patients.size());
		}

		baseTable.setContainerDataSource(null);
		for (Patient patient : patients) {
			patient.setDisplayName();
			Item item = indexedContainer.addItem(patient.getId());
			item.getItemProperty("multiSelect").setValue(Boolean.FALSE);
			item.getItemProperty("patientId").setValue(patient.getPatientId());
			item.getItemProperty("userName").setValue(patient.getUserName());
			item.getItemProperty("doctor").setValue(patient.getDoctor() == null ? "" : patient.getDoctor().getDisplayName());
			item.getItemProperty("patientStatus").setValue(patient.getPatientStatus());
			item.getItemProperty("isVisible").setValue(Boolean.TRUE);
			if (patientService.userHasPermission(PermissionName.PERM_PATIENT_UPDATE_ANY)) {
				item.getItemProperty("isEditCellEnabled").setValue(Boolean.TRUE);
			}
			if (patientService.userHasPermission(PermissionName.PERM_PATIENT_DELETE_ANY)) {
				item.getItemProperty("isRemoveCellEnabled").setValue(Boolean.TRUE);
			}
		}
		baseTable.setContainerDataSource(indexedContainer);
		setVisibleColumns(visibleColumns.toArray());
	}

	@Override
	protected void setBaseTable() {
		baseTable = new BaseTable(null, indexedContainer);
		baseTable.setCacheRate(2);

		if (filterInfoVerticalLayout.getComponentCount() != 0) {
			filterInfoVerticalLayout = new VerticalLayout();
			patientFilter = new PatientFilter();
		}

		filterInfoVerticalLayout.setImmediate(true);
		filterInfoVerticalLayout.setSizeFull();
		filterInfoVerticalLayout.addComponent(baseTable);
		filterInfoVerticalLayout.setExpandRatio(baseTable, 1);
		addComponent(filterInfoVerticalLayout);
		setExpandRatio(filterInfoVerticalLayout, 1);
	}

	@Override
	public void windowClose(Window.CloseEvent closeEvent) {

	}

	private Button getSelectAllButton() {
		selectAllButton.setIcon(FontAwesome.CHECK_SQUARE);
		selectAllButton.addStyleName(ValoTheme.BUTTON_SMALL);
		selectAllButton.addStyleName(ValoTheme.BUTTON_ICON_ONLY);
		selectAllButton.addClickListener(selectAllButtonClickListener);

		return selectAllButton;
	}

	private ComboBox getBulkActionComboBox() {
		bulkActionComboBox.setWidth(160, Unit.PIXELS);
		bulkActionComboBox.addStyleName(ValoTheme.COMBOBOX_SMALL);
		bulkActionComboBox.setNullSelectionAllowed(false);
		bulkActionComboBox.setTextInputAllowed(false);
		bulkActionComboBox.setInputPrompt(i18n.get("GridView.bulkActionLabel"));

		bulkActionComboBox.addItem(BulkAction.DOCTOR);
		bulkActionComboBox.setItemCaption(BulkAction.DOCTOR, BulkAction.DOCTOR.getCaption());
		bulkActionComboBox.addItem(BulkAction.SERVICE_GROUP);
		bulkActionComboBox.setItemCaption(BulkAction.SERVICE_GROUP, BulkAction.SERVICE_GROUP.getCaption());

		if (UserUtil.isAssistant()) {
			bulkActionComboBox.addItem(BulkAction.MESSAGE_BROADCAST);
			bulkActionComboBox.setItemCaption(BulkAction.MESSAGE_BROADCAST, BulkAction.MESSAGE_BROADCAST.getCaption());
		}

		bulkActionComboBox.addValueChangeListener(bulkActionComboBoxValueChangeListener);

		return bulkActionComboBox;
	}

	private Component getQuickSearchComponent() {
		quickSearchTextField.addStyleName(ValoTheme.TEXTFIELD_SMALL);
		quickSearchTextField.setInputPrompt(i18n.get("BaseForm.quickSearch"));
		quickSearchTextField.addValueChangeListener(quickSearchValueChangeListener);

		return quickSearchTextField;
	}

	private ComboBox getFilterComboBox() {
		filterComboBox.setWidth(90, Unit.PIXELS);
		filterComboBox.addStyleName(ValoTheme.COMBOBOX_SMALL);
		filterComboBox.setNullSelectionAllowed(false);
		filterComboBox.setTextInputAllowed(false);
		filterComboBox.setInputPrompt(i18n.get("BaseView.filterLabel"));
		setFilterComboBoxItems();
		filterComboBox.addValueChangeListener(filterComboBoxValueChangeListener);

		return filterComboBox;
	}

	private void setFilterComboBoxItems() {
		filterComboBox.removeValueChangeListener(filterComboBoxValueChangeListener);
		filterComboBox.removeAllItems();
		for (PatientFilter patientFilter : filterService.findAllByUser(UserUtil.getUser(), FilterCategory.PATIENT, PatientFilter.class)) {
			filterComboBox.addItem(patientFilter);
			filterComboBox.setItemCaption(patientFilter, patientFilter.getFilterName());
		}

		/**
		 * Global Filters
		 */
		if (UserUtil.isDoctor()) {
			String enabledGlobalFilters = UserUtil.getUser().getEnabledGlobalFilters();
			if (enabledGlobalFilters == null) {
				enabledGlobalFilters = StringUtils.join(GlobalFilter.values(), ",");
			}

			if (!enabledGlobalFilters.isEmpty()) {
				for (String globalFilterString : enabledGlobalFilters.split(",")) {
					GlobalFilter globalFilter = GlobalFilter.valueOf(globalFilterString);
					filterComboBox.addItem(globalFilter);
					filterComboBox.setItemCaption(globalFilter, globalFilter.getCaption());
				}
			}
		}
		filterComboBox.addItem("showFilter");
		filterComboBox.setItemCaption("showFilter", i18n.get("BaseView.showFilterLabel"));
		filterComboBox.addValueChangeListener(filterComboBoxValueChangeListener);
	}

	private ComboBox getOtherComboBox() {
		otherComboBox.setWidth(90, Unit.PIXELS);
		otherComboBox.addStyleName(ValoTheme.COMBOBOX_SMALL);
		otherComboBox.setNullSelectionAllowed(false);
		otherComboBox.setTextInputAllowed(false);
		otherComboBox.setInputPrompt(i18n.get("GridView.otherLabel"));

		otherComboBox.addItem(BulkAction.EXPORT);
		otherComboBox.setItemCaption(BulkAction.EXPORT, i18n.get("GridView.otherExportLabel"));

		otherComboBox.addValueChangeListener(otherComboBoxValueChangeListener);

		return otherComboBox;
	}

	@EventBusListenerMethod
	public void buttonClick(EventBusAction eventBusAction) {
		if (eventBusAction.getFilter() != null && PatientFilter.class.getName().equals(eventBusAction.getFilter().getClass().getName())) {
			if (eventBusAction.getAction().equals(Action.FILTER)) {
				patientFilter = (PatientFilter) eventBusAction.getFilter();
				refreshTable();
			} else if (eventBusAction.getAction().equals(Action.DELETE_FILTER)) {
				quickSearchTextField.removeValueChangeListener(quickSearchValueChangeListener);
				patientFilter = (PatientFilter) eventBusAction.getFilter();
				quickSearchTextField.setValue("");
				quickSearchTextField.addValueChangeListener(quickSearchValueChangeListener);
				refreshTable();
			} else if (eventBusAction.getAction().equals(Action.SAVE_FILTER)) {
				setFilterComboBoxItems();
			}
		}
	}
}
