package hu.oroscom.c1801.view;

import com.vaadin.data.Property;
import com.vaadin.data.util.IndexedContainer;
import com.vaadin.server.FontAwesome;
import com.vaadin.server.Page;
import com.vaadin.ui.*;
import com.vaadin.ui.themes.ValoTheme;
import hu.oroscom.c1801.NeededAnyPermission;
import hu.oroscom.c1801.component.datefield.JodaDateField;
import hu.oroscom.c1801.component.form.ReportForm;
import hu.oroscom.c1801.component.table.StandaloneTable;
import hu.oroscom.c1801.component.tabsheet.ReportTabPage;
import hu.oroscom.c1801.converter.DateToJodaDateConverter;
import hu.oroscom.c1801.domain.Report;
import hu.oroscom.c1801.enumeration.PermissionName;
import hu.oroscom.c1801.enumeration.ReportCategory;
import hu.oroscom.c1801.service.ExcelExportLogService;
import hu.oroscom.c1801.service.ReportService;
import hu.oroscom.c1801.service.SettingService;
import hu.oroscom.c1801.util.LoggingExcelExport;
import org.joda.time.DateTime;
import org.vaadin.spring.annotation.VaadinUIScope;
import org.vaadin.spring.navigator.annotation.VaadinView;

import javax.inject.Inject;

@VaadinUIScope
@VaadinView(name = ReportsView.NAME)
@NeededAnyPermission(values = {
		PermissionName.PERM_REPORT_SELECT_ANY
})
public class ReportsView extends BaseTabSheetView<Report, ReportService, ReportForm> {
	public static final String NAME = "reports";

	private StandaloneTable standaloneTable;
	private IndexedContainer indexedContainer;
	private DateTime fromDateTime = new DateTime();
	private DateTime toDateTime = new DateTime();
	private ProgressBar dataLoadingProgressBar;
	private Button showReportButton;
	private Button printReportButton;

	@Inject
	private ReportService reportService;

	@Inject
	private SettingService settingService;

	@Inject
	private ExcelExportLogService excelExportLogService;

	@Override
	public void windowClose(Window.CloseEvent closeEvent) {

	}

	@Override
	protected void registerColumns() {

	}

	@Override
	protected void init() {
		tabSheet = new TabSheet();
		tabSheet.setSizeFull();

		for (ReportCategory reportCategory : ReportCategory.values()) {
			ReportTabPage reportTabPage = new ReportTabPage(reportCategory);
			tabSheet.addTab(reportTabPage, reportCategory.getCaption());
		}

		tabSheet.addSelectedTabChangeListener(new TabSheet.SelectedTabChangeListener() {
			@Override
			public void selectedTabChange(TabSheet.SelectedTabChangeEvent event) {
				setDataSource();
			}
		});

		addComponent(tabSheet);
		setExpandRatio(tabSheet, 1);

		setDataSource();

		new Notification("Riportszerver hiba!", "A riportszerver nem válaszol. Hibakód: EA142", Notification.Type.ERROR_MESSAGE, true).show(Page.getCurrent());
	}

	@Override
	protected void setDataSource() {
		ReportTabPage reportTabPage = (ReportTabPage) tabSheet.getSelectedTab();
		if (reportTabPage == null) {
			return;
		}
		reportTabPage.removeAllComponents();

		generateReportTabHeader(reportTabPage);
		generateReportTabBody(reportTabPage);
	}

	private void generateReportTabHeader(final ReportTabPage reportTabPage) {
		HorizontalLayout reportTabHeaderHorizontalLayout = new HorizontalLayout();
		reportTabHeaderHorizontalLayout.setSpacing(true);
		reportTabHeaderHorizontalLayout.addStyleName("report-tab-header");

		HorizontalLayout reportTabSecondHeaderHorizontalLayout = new HorizontalLayout();
		reportTabSecondHeaderHorizontalLayout.setSpacing(true);
		reportTabSecondHeaderHorizontalLayout.addStyleName("report-tab-header");

		final JodaDateField startJodaDateField = new JodaDateField(i18n.get("ReportsView.startDate"));
		startJodaDateField.setConverter(new DateToJodaDateConverter());
		startJodaDateField.addStyleName(ValoTheme.TEXTFIELD_TINY);
		startJodaDateField.addStyleName("start");
		startJodaDateField.setJodaValue(fromDateTime);
		startJodaDateField.addValueChangeListener(new Property.ValueChangeListener() {
			@Override
			public void valueChange(Property.ValueChangeEvent event) {
				fromDateTime = startJodaDateField.getJodaValue();
			}
		});
		reportTabHeaderHorizontalLayout.addComponent(prepareComponentToHeader(startJodaDateField));

		final JodaDateField endJodaDateField = new JodaDateField(i18n.get("ReportsView.endDate"));
		endJodaDateField.setConverter(new DateToJodaDateConverter());
		endJodaDateField.addStyleName(ValoTheme.TEXTFIELD_TINY);
		endJodaDateField.addStyleName("end");
		endJodaDateField.setJodaValue(toDateTime);
		endJodaDateField.addValueChangeListener(new Property.ValueChangeListener() {
			@Override
			public void valueChange(Property.ValueChangeEvent event) {
				toDateTime = endJodaDateField.getJodaValue();
			}
		});
		reportTabHeaderHorizontalLayout.addComponent(prepareComponentToHeader(endJodaDateField));

		showReportButton = new Button(FontAwesome.PLAY);
		showReportButton.addStyleName(ValoTheme.BUTTON_ICON_ONLY);
		showReportButton.addStyleName("report-button");
		showReportButton.addStyleName(ValoTheme.BUTTON_BORDERLESS);
		showReportButton.addClickListener(new Button.ClickListener() {
			@Override
			public void buttonClick(Button.ClickEvent event) {
				if (startJodaDateField.getJodaValue() != null && endJodaDateField.getJodaValue() != null) {
						refreshTable(reportTabPage, startJodaDateField.getJodaValue(), endJodaDateField.getJodaValue());
				}
			}
		});
		reportTabHeaderHorizontalLayout.addComponent(prepareComponentToHeader(showReportButton));

		printReportButton = new Button(FontAwesome.PRINT);
		printReportButton.addStyleName(ValoTheme.BUTTON_ICON_ONLY);
		printReportButton.addStyleName("report-button");
		printReportButton.addStyleName(ValoTheme.BUTTON_BORDERLESS);
		printReportButton.addClickListener(new Button.ClickListener() {
			@Override
			public void buttonClick(Button.ClickEvent event) {
				boolean isColumnHeaderTitleAllowed = standaloneTable.isColumnHeaderTitleAllowed();
				standaloneTable.setColumnHeaderTitleAllowed(false);
				LoggingExcelExport excelExport = new LoggingExcelExport(standaloneTable, reportTabPage.getReportCategory().getCaption(), reportTabPage.getReportCategory().getCaption(), reportTabPage.getReportCategory().name() + ".xls", false, excelExportLogService, i18n.get("MainUI.reportsMenuItem") + " - " + reportTabPage.getReportCategory().getCaption());
				excelExport.excludeCollapsedColumns();
				for (Object o : indexedContainer.getContainerPropertyIds()) {
					switch (reportTabPage.getReportCategory()) {

					}
				}
				excelExport.setDoubleDataFormat("#,##0");
				excelExport.export();
				standaloneTable.setColumnHeaderTitleAllowed(isColumnHeaderTitleAllowed);
			}
		});
		reportTabHeaderHorizontalLayout.addComponent(prepareComponentToHeader(printReportButton));

		dataLoadingProgressBar = new ProgressBar();
		dataLoadingProgressBar.setVisible(false);
		dataLoadingProgressBar.setIndeterminate(true);
		reportTabHeaderHorizontalLayout.addComponent(prepareComponentToHeader(dataLoadingProgressBar));

		reportTabPage.addComponent(reportTabHeaderHorizontalLayout);
		reportTabPage.addComponent(reportTabSecondHeaderHorizontalLayout);
	}

	private Component prepareComponentToHeader(Component component) {
		FormLayout formLayout = new FormLayout(component);
		formLayout.setMargin(false);
		formLayout.addStyleName("no-errorcell");
		return formLayout;
	}

	private void generateReportTabBody(final ReportTabPage reportTabPage) {
		indexedContainer = new IndexedContainer();

		standaloneTable = new StandaloneTable();
		standaloneTable.setHeight(100, Unit.PERCENTAGE);
		standaloneTable.addStyleName(ValoTheme.TABLE_SMALL);
		standaloneTable.setColumnCollapsingAllowed(true);
		standaloneTable.removeStyleName(ValoTheme.TABLE_BORDERLESS);
		standaloneTable.setColumnHeaderMode(Table.ColumnHeaderMode.EXPLICIT_DEFAULTS_ID);

		indexedContainer.addContainerProperty("rowTitle", String.class, "");
		standaloneTable.setColumnHeader("rowTitle", "");

		switch (reportTabPage.getReportCategory()) {

		}

		standaloneTable.setCellStyleGenerator(new Table.CellStyleGenerator() {
			@Override
			public String getStyle(Table source, Object itemId, Object propertyId) {
				return null;
			}
		});

		reportTabPage.addComponent(standaloneTable);
		reportTabPage.setExpandRatio(standaloneTable, 1);
	}


	private void refreshTable(ReportTabPage reportTabPage, DateTime startDateTime, DateTime endDateTime) {
		refreshTable(reportTabPage, startDateTime, endDateTime, null, null, null, null, null);
	}

	private void refreshTable(ReportTabPage reportTabPage, DateTime startDateTime, DateTime endDateTime, Object venueComboBoxValue, Object resourceCategoryComboBoxValue, Object parentResourceTypeComboBoxValue, Object childResourceTypeComboBoxValue, Object resourceComboBoxValue) {
		startLoadingData();
		standaloneTable.setContainerDataSource(null);
		indexedContainer.removeAllItems();

		switch (reportTabPage.getReportCategory()) {

		}

		standaloneTable.setContainerDataSource(indexedContainer);
	}

	private void startLoadingData() {
		dataLoadingProgressBar.setVisible(true);
		showReportButton.setEnabled(false);
		printReportButton.setEnabled(false);
	}

	private void stopLoadingData() {
		dataLoadingProgressBar.setVisible(false);
		showReportButton.setEnabled(true);
		printReportButton.setEnabled(true);
	}

}
