package hu.oroscom.c1801.view;

import com.vaadin.data.Item;
import com.vaadin.data.Property;
import com.vaadin.data.util.BeanItemContainer;
import com.vaadin.server.VaadinSession;
import com.vaadin.ui.*;
import com.vaadin.ui.themes.ValoTheme;
import hu.oroscom.c1801.NeededAnyPermission;
import hu.oroscom.c1801.component.FilterInfo;
import hu.oroscom.c1801.component.cell.*;
import hu.oroscom.c1801.component.form.ServiceManagementForm;
import hu.oroscom.c1801.component.table.BaseTable;
import hu.oroscom.c1801.component.windowed.FilterPanel;
import hu.oroscom.c1801.domain.ServiceManagement;
import hu.oroscom.c1801.domain.User;
import hu.oroscom.c1801.domain.filter.ServiceManagementFilter;
import hu.oroscom.c1801.enumeration.*;
import hu.oroscom.c1801.service.ExcelExportLogService;
import hu.oroscom.c1801.service.FilterService;
import hu.oroscom.c1801.service.ServiceManagementService;
import hu.oroscom.c1801.service.UserService;
import hu.oroscom.c1801.util.EventBusAction;
import hu.oroscom.c1801.util.FilterUtil;
import hu.oroscom.c1801.util.LoggingExcelExport;
import hu.oroscom.c1801.util.UserUtil;
import org.vaadin.spring.annotation.VaadinUIScope;
import org.vaadin.spring.events.annotation.EventBusListenerMethod;
import org.vaadin.spring.navigator.annotation.VaadinView;

import javax.inject.Inject;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

/**
 * Created by kollerk on 14. 11. 12..
 */
@VaadinUIScope
@VaadinView(name = ServiceManagementView.NAME)
@NeededAnyPermission(values = {
		PermissionName.PERM_SERVICE_MANAGEMENT_SELECT_ANY,
})
public class ServiceManagementView extends GridView<ServiceManagement, ServiceManagementService, ServiceManagementForm> {
	public static final String NAME = "serviceManagement";
	private final List<Object> visibleColumns = new ArrayList<>(Arrays.asList(new Object[]{"name", "serviceManagementNature", "serviceManagementStatus", "user", "studyEntryType", "remove"}));

	@Inject
	private ServiceManagementService serviceManagementService;

	@Inject
	private FilterService filterService;

	@Inject
	private ExcelExportLogService excelExportLogService;

	@Inject
	private UserService userService;

	private ComboBox otherComboBox = new ComboBox();
	private ComboBox filterComboBox = new ComboBox();
	private TextField quickSearchTextField = new TextField();
	private ServiceManagementFilter serviceManagementFilter = new ServiceManagementFilter();
	private VerticalLayout filterInfoVerticalLayout = new VerticalLayout();
	private FilterInfo filterInfo;

	private final Property.ValueChangeListener filterComboBoxValueChangeListener = new Property.ValueChangeListener() {
		@Override
		public void valueChange(Property.ValueChangeEvent valueChangeEvent) {
			if ("showFilter".equals(valueChangeEvent.getProperty().getValue())) {
				FilterPanel filterPanel = new FilterPanel(serviceManagementFilter, filterService, serviceManagementService, userService);
				addContentToWindow(filterPanel);
			} else {
				serviceManagementFilter = (ServiceManagementFilter) valueChangeEvent.getProperty().getValue();
				refreshTable();
			}

			filterComboBox.removeValueChangeListener(filterComboBoxValueChangeListener);
			filterComboBox.setValue(null);
			filterComboBox.addValueChangeListener(filterComboBoxValueChangeListener);
		}
	};

	private final Property.ValueChangeListener quickSearchValueChangeListener = new Property.ValueChangeListener() {
		@Override
		public void valueChange(Property.ValueChangeEvent event) {
			String searchText = event.getProperty().getValue().toString();
			if (searchText.length() > 2) {
				serviceManagementFilter.setQuickSearch(searchText);
				eventBus.publish(this, new EventBusAction<>(Action.FILTER, serviceManagementFilter));
			} else {
				Notification.show(i18n.get("QuickSearch.minThreeCharacter"), Notification.Type.WARNING_MESSAGE);
			}
		}
	};

	private final Property.ValueChangeListener otherComboBoxValueChangeListener = new Property.ValueChangeListener() {
		@Override
		public void valueChange(Property.ValueChangeEvent valueChangeEvent) {
			final BulkAction bulkAction = (BulkAction) valueChangeEvent.getProperty().getValue();
			switch (bulkAction) {
				case EXPORT:
					LoggingExcelExport excelExport = new LoggingExcelExport(baseTable, i18n.get("ServiceManagementView.serviceManagementLabel"), i18n.get("ServiceManagementView.serviceManagementLabel"), "serviceManagement.xls", false, excelExportLogService, i18n.get("ServiceManagementView.serviceManagementLabel"));
					excelExport.excludeCollapsedColumns();
					Iterator iterator = excelExport.getPropIds().iterator();

					while (iterator.hasNext()) {
						Object propId = iterator.next();
						if ("remove".equals(propId)) {
							iterator.remove();
						}
					}
					excelExport.export();
					break;
			}

			otherComboBox.removeValueChangeListener(otherComboBoxValueChangeListener);
			otherComboBox.setValue(null);
			otherComboBox.addValueChangeListener(otherComboBoxValueChangeListener);
		}
	};

	@Override
	protected void init() {
		super.defaultComponents(i18n.get("ServiceManagementView.serviceManagementLabel"));
		initTable();
	}

	@Override
	protected void customizeView() {
		if (serviceManagementService.userHasPermission(PermissionName.PERM_SERVICE_MANAGEMENT_CREATE_ANY)) {
			setAddButton(i18n.get("ServiceManagementView.addServiceManagementButton"));
		}

		Label expandingGap = new Label();
		expandingGap.setWidth("100%");

		getHeaderToolBarHorizontalLayout().setWidth(100, Unit.PERCENTAGE);
		getHeaderToolBarHorizontalLayout().setSpacing(true);
		getHeaderToolBarHorizontalLayout().addComponent(getQuickSearchComponent());
		getHeaderToolBarHorizontalLayout().addComponent(getFilterComboBox());
		getHeaderToolBarHorizontalLayout().addComponent(expandingGap);
		getHeaderToolBarHorizontalLayout().addComponent(getOtherComboBox());
		getHeaderToolBarHorizontalLayout().setExpandRatio(expandingGap, 1);
	}

	@Override
	protected void registerColumns() {
		baseTable.addStyleName("serviceManagement-view");
		baseTable.setHeight(100, Unit.PERCENTAGE);
		baseTable.addStyleName(ValoTheme.TABLE_SMALL);
		baseTable.setColumnCollapsingAllowed(true);
		baseTable.setColumnReorderingAllowed(true);

		addGeneratedColumn("name", new EditCell(editClickListener));
		addGeneratedColumn("serviceManagementNature", new ServiceManagementNatureCell());
		addGeneratedColumn("serviceManagementStatus", new ServiceManagementStatusCell());
		addGeneratedColumn("user", new UserCell());
		addGeneratedColumn("studyEntryType", new StudyEntryTypeCell());
		addGeneratedColumn("remove", new RemoveCell<>(ServiceManagementService.class));

		addContainerProperty("id", Long.class, 0L);
		addContainerProperty("name", String.class, "");
		addContainerProperty("serviceManagementNature", ServiceManagementNature.class, null);
		addContainerProperty("serviceManagementStatus", ServiceManagementStatus.class, ServiceManagementStatus.NEW);
		addContainerProperty("user", User.class, null);
		addContainerProperty("studyEntryType", StudyEntryType.class, null);
		addContainerProperty("isRemoveCellEnabled", Boolean.class, false);
		addContainerProperty("isEditCellEnabled", Boolean.class, false);
		addContainerProperty("isEditCalendarVenueListCellEnabled", Boolean.class, false);

		baseTable.setColumnHeader("name", i18n.get("ServiceManagementView.id"));
		baseTable.setColumnHeader("serviceManagementNature", i18n.get("ServiceManagementView.serviceManagementNature"));
		baseTable.setColumnHeader("serviceManagementStatus", i18n.get("ServiceManagementView.serviceManagementStatus"));
		baseTable.setColumnHeader("user", i18n.get("ServiceManagementView.user"));
		baseTable.setColumnHeader("studyEntryType", i18n.get("ServiceManagementView.studyEntryType"));
		baseTable.setColumnHeader("remove", "");
	}

	@Override
	protected ServiceManagement getFormEntity() {
		return new ServiceManagement();
	}

	@Override
	protected void generateForm(ServiceManagement tEntity) {
		BeanItemContainer<User> userBeanItemContainer = new BeanItemContainer<>(User.class, userService.findAllByCustomer(RoleName.ROLE_SUPER_ADMIN));
		form = new ServiceManagementForm(tEntity, true, userBeanItemContainer);
	}

	@Override
	protected void setGridDataSource() {
		ServiceManagementFilter sessionServiceManagementFilter = VaadinSession.getCurrent().getAttribute(ServiceManagementFilter.class);
		if (sessionServiceManagementFilter != null) {
			serviceManagementFilter = sessionServiceManagementFilter;
			VaadinSession.getCurrent().setAttribute(ServiceManagementFilter.class, null);
		}

		if (FilterUtil.isFilterActive(serviceManagementFilter)) {
			try {
				filterInfoVerticalLayout.removeComponent(filterInfo);
			} catch (Exception e) {
			}
			filterInfo = new FilterInfo<>(serviceManagementFilter, userService);
			filterInfoVerticalLayout.addComponent(filterInfo, 0);
		} else {
			try {
				quickSearchTextField.setValue("");
				filterInfoVerticalLayout.removeComponent(filterInfo);
			} catch (Exception e) {
			}
		}

		List<ServiceManagement> serviceManagements = serviceManagementService.findAllWithFilter(serviceManagementFilter);
		if (filterInfo != null) {
			filterInfo.setEntityCount(serviceManagements.size());
		}

		DecimalFormat decimalFormat = new DecimalFormat("000000");
		baseTable.setContainerDataSource(null);
		for (ServiceManagement serviceManagement : serviceManagements) {
			Item item = indexedContainer.addItem(serviceManagement.getId());
			item.getItemProperty("id").setValue(serviceManagement.getId());
			item.getItemProperty("name").setValue(decimalFormat.format(serviceManagement.getId()));
			item.getItemProperty("serviceManagementNature").setValue(serviceManagement.getServiceManagementNature());
			item.getItemProperty("serviceManagementStatus").setValue(serviceManagement.getServiceManagementStatus());
			item.getItemProperty("user").setValue(serviceManagement.getUser());
			item.getItemProperty("studyEntryType").setValue(serviceManagement.getStudyEntryType());
			if (serviceManagementService.userHasPermission(PermissionName.PERM_SERVICE_MANAGEMENT_DELETE_ANY)) {
				item.getItemProperty("isRemoveCellEnabled").setValue(Boolean.TRUE);
			}
			if (serviceManagementService.userHasPermission(PermissionName.PERM_SERVICE_MANAGEMENT_UPDATE_ANY)) {
				item.getItemProperty("isEditCellEnabled").setValue(Boolean.TRUE);
			}
		}
		baseTable.setContainerDataSource(indexedContainer);
		setVisibleColumns(visibleColumns.toArray());
	}

	@Override
	protected void setBaseTable() {
		baseTable = new BaseTable(null, indexedContainer);
		baseTable.setCacheRate(2);

		if (filterInfoVerticalLayout.getComponentCount() != 0) {
			filterInfoVerticalLayout = new VerticalLayout();
			serviceManagementFilter = new ServiceManagementFilter();
		}

		filterInfoVerticalLayout.setImmediate(true);
		filterInfoVerticalLayout.setSizeFull();
		filterInfoVerticalLayout.addComponent(baseTable);
		filterInfoVerticalLayout.setExpandRatio(baseTable, 1);
		addComponent(filterInfoVerticalLayout);
		setExpandRatio(filterInfoVerticalLayout, 1);
	}

	@Override
	public void windowClose(Window.CloseEvent closeEvent) {

	}

	private Component getQuickSearchComponent() {
		quickSearchTextField.addStyleName(ValoTheme.TEXTFIELD_SMALL);
		quickSearchTextField.setInputPrompt(i18n.get("BaseForm.quickSearch"));
		quickSearchTextField.addValueChangeListener(quickSearchValueChangeListener);

		return quickSearchTextField;
	}

	private ComboBox getFilterComboBox() {
		filterComboBox.setWidth(90, Unit.PIXELS);
		filterComboBox.addStyleName(ValoTheme.COMBOBOX_SMALL);
		filterComboBox.setNullSelectionAllowed(false);
		filterComboBox.setTextInputAllowed(false);
		filterComboBox.setInputPrompt(i18n.get("BaseView.filterLabel"));
		setFilterComboBoxItems();
		filterComboBox.addValueChangeListener(filterComboBoxValueChangeListener);

		return filterComboBox;
	}

	private void setFilterComboBoxItems() {
		filterComboBox.removeAllItems();
		for (ServiceManagementFilter serviceManagementFilter : filterService.findAllByUser(UserUtil.getUser(), FilterCategory.SERVICE_MANAGEMENT, ServiceManagementFilter.class)) {
			filterComboBox.addItem(serviceManagementFilter);
			filterComboBox.setItemCaption(serviceManagementFilter, serviceManagementFilter.getFilterName());
		}

		filterComboBox.addItem("showFilter");
		filterComboBox.setItemCaption("showFilter", i18n.get("BaseView.showFilterLabel"));
	}

	private ComboBox getOtherComboBox() {
		otherComboBox.setWidth(90, Unit.PIXELS);
		otherComboBox.addStyleName(ValoTheme.COMBOBOX_SMALL);
		otherComboBox.setNullSelectionAllowed(false);
		otherComboBox.setTextInputAllowed(false);
		otherComboBox.setInputPrompt(i18n.get("GridView.otherLabel"));

		otherComboBox.addItem(BulkAction.EXPORT);
		otherComboBox.setItemCaption(BulkAction.EXPORT, i18n.get("GridView.otherExportLabel"));

		otherComboBox.addValueChangeListener(otherComboBoxValueChangeListener);

		return otherComboBox;
	}

	@EventBusListenerMethod
	public void buttonClick(EventBusAction eventBusAction) {
		if (eventBusAction.getFilter() != null && ServiceManagementFilter.class.getName().equals(eventBusAction.getFilter().getClass().getName())) {
			if (eventBusAction.getAction().equals(Action.FILTER)) {
				serviceManagementFilter = (ServiceManagementFilter) eventBusAction.getFilter();
				refreshTable();
			} else if (eventBusAction.getAction().equals(Action.DELETE_FILTER)) {
				quickSearchTextField.removeValueChangeListener(quickSearchValueChangeListener);
				serviceManagementFilter = (ServiceManagementFilter) eventBusAction.getFilter();
				quickSearchTextField.setValue("");
				quickSearchTextField.addValueChangeListener(quickSearchValueChangeListener);
				refreshTable();
			} else if (eventBusAction.getAction().equals(Action.SAVE_FILTER)) {
				setFilterComboBoxItems();
			}
		}
	}
}
