package hu.oroscom.c1801.view;

import com.vaadin.data.Item;
import com.vaadin.data.Property;
import com.vaadin.server.VaadinSession;
import com.vaadin.ui.*;
import com.vaadin.ui.themes.ValoTheme;
import hu.oroscom.c1801.NeededAnyPermission;
import hu.oroscom.c1801.component.FilterInfo;
import hu.oroscom.c1801.component.cell.DateTimeFormatCell;
import hu.oroscom.c1801.component.cell.EditLinkCell;
import hu.oroscom.c1801.component.cell.EditStudyEntryTypeCell;
import hu.oroscom.c1801.component.cell.RemoveCell;
import hu.oroscom.c1801.component.form.StudyEntryForm;
import hu.oroscom.c1801.component.table.BaseTable;
import hu.oroscom.c1801.component.windowed.FilterPanel;
import hu.oroscom.c1801.domain.Patient;
import hu.oroscom.c1801.domain.StudyEntry;
import hu.oroscom.c1801.domain.filter.StudyEntryFilter;
import hu.oroscom.c1801.enumeration.*;
import hu.oroscom.c1801.service.ExcelExportLogService;
import hu.oroscom.c1801.service.FilterService;
import hu.oroscom.c1801.service.StudyEntryService;
import hu.oroscom.c1801.util.EventBusAction;
import hu.oroscom.c1801.util.FilterUtil;
import hu.oroscom.c1801.util.LoggingExcelExport;
import hu.oroscom.c1801.util.UserUtil;
import org.joda.time.DateTime;
import org.vaadin.spring.annotation.VaadinUIScope;
import org.vaadin.spring.events.annotation.EventBusListenerMethod;
import org.vaadin.spring.navigator.annotation.VaadinView;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

@VaadinUIScope
@VaadinView(name = StudyEntriesView.NAME)
@NeededAnyPermission(values = {
		PermissionName.PERM_STUDY_ENTRY_SELECT_ANY
})
public class StudyEntriesView extends GridView<StudyEntry, StudyEntryService, StudyEntryForm> {
	public static final String NAME = "studyEntries";
	private final List<Object> visibleColumns = new ArrayList<>(Arrays.asList(new Object[]{"studyEntryType", "time", "studyEntryStudyEntryData", "description", "remove"}));
	private Patient patient;
	private ComboBox otherComboBox = new ComboBox();
	private ComboBox filterComboBox = new ComboBox();
	private TextField quickSearchTextField = new TextField();
	private StudyEntryFilter studyEntryFilter = new StudyEntryFilter();
	private VerticalLayout filterInfoVerticalLayout = new VerticalLayout();
	private FilterInfo filterInfo;

	@Inject
	private StudyEntryService studyEntryService;

	@Inject
	private FilterService filterService;

	@Inject
	private ExcelExportLogService excelExportLogService;

	private final Property.ValueChangeListener filterComboBoxValueChangeListener = new Property.ValueChangeListener() {
		@Override
		public void valueChange(Property.ValueChangeEvent valueChangeEvent) {
			if ("showFilter".equals(valueChangeEvent.getProperty().getValue())) {
				FilterPanel filterPanel = new FilterPanel(studyEntryFilter, filterService, studyEntryService);
				addContentToWindow(filterPanel);
			} else {
				studyEntryFilter = (StudyEntryFilter) valueChangeEvent.getProperty().getValue();
				refreshTable();
			}

			filterComboBox.removeValueChangeListener(filterComboBoxValueChangeListener);
			filterComboBox.setValue(null);
			filterComboBox.addValueChangeListener(filterComboBoxValueChangeListener);
		}
	};

	private final Property.ValueChangeListener quickSearchValueChangeListener = new Property.ValueChangeListener() {
		@Override
		public void valueChange(Property.ValueChangeEvent event) {
			String searchText = event.getProperty().getValue().toString();
			if (searchText.length() > 2) {
				studyEntryFilter.setQuickSearch(searchText);
				eventBus.publish(this, new EventBusAction<>(Action.FILTER, studyEntryFilter));
			} else {
				Notification.show(i18n.get("QuickSearch.minThreeCharacter"), Notification.Type.WARNING_MESSAGE);
			}
		}
	};

	private final Property.ValueChangeListener otherComboBoxValueChangeListener = new Property.ValueChangeListener() {
		@Override
		public void valueChange(Property.ValueChangeEvent valueChangeEvent) {
			final BulkAction bulkAction = (BulkAction) valueChangeEvent.getProperty().getValue();
			switch (bulkAction) {
				case EXPORT:
					LoggingExcelExport excelExport = new LoggingExcelExport(baseTable, i18n.get("StudyEntryView.heading"), i18n.get("StudyEntryView.heading"), "studyEntry.xls", false, excelExportLogService, i18n.get("StudyEntryView.heading"));
					excelExport.excludeCollapsedColumns();
					Iterator iterator = excelExport.getPropIds().iterator();

					while (iterator.hasNext()) {
						Object propId = iterator.next();
						if ("remove".equals(propId)) {
							iterator.remove();
						}
					}
					excelExport.export();
					break;
			}

			otherComboBox.removeValueChangeListener(otherComboBoxValueChangeListener);
			otherComboBox.setValue(null);
			otherComboBox.addValueChangeListener(otherComboBoxValueChangeListener);
		}
	};

	private Button.ClickListener studyEntryDataClickListener = new Button.ClickListener() {
		@Override
		public void buttonClick(Button.ClickEvent event) {
			StudyEntry studyEntry = studyEntryService.findOneWithRelationTable((Long) event.getButton().getData());
			addFormToWindow(new StudyEntryForm(studyEntry, true, true));
		}
	};

	@Override
	protected void init() {
		super.defaultComponents(i18n.get("StudyEntriesView.heading"));
		initTable();
	}

	@Override
	protected void customizeView() {
		if (!(UserUtil.getUser() instanceof Patient)) {
			setBackButton(PatientsView.NAME);
		}

		Label expandingGap = new Label();
		expandingGap.setWidth("100%");

		getHeaderToolBarHorizontalLayout().setWidth(100, Unit.PERCENTAGE);
		getHeaderToolBarHorizontalLayout().setSpacing(true);
		getHeaderToolBarHorizontalLayout().addComponent(getQuickSearchComponent());
		getHeaderToolBarHorizontalLayout().addComponent(getFilterComboBox());
		getHeaderToolBarHorizontalLayout().addComponent(expandingGap);
		getHeaderToolBarHorizontalLayout().addComponent(getOtherComboBox());
		getHeaderToolBarHorizontalLayout().setExpandRatio(expandingGap, 1);

		if (studyEntryService.userHasPermission(PermissionName.PERM_STUDY_ENTRY_CREATE_ANY)) {
			setAddButton(i18n.get("StudyEntriesView.addButton"));
		}
	}

	@Override
	protected void registerColumns() {
		baseTable.addStyleName("studyEntriesView-view");
		baseTable.setHeight(100, Unit.PERCENTAGE);
		baseTable.addStyleName(ValoTheme.TABLE_SMALL);
		baseTable.setColumnCollapsingAllowed(true);
		baseTable.setColumnReorderingAllowed(true);

		addGeneratedColumn("studyEntryType", new EditStudyEntryTypeCell(editClickListener));
		addGeneratedColumn("time", new DateTimeFormatCell(true));
		addGeneratedColumn("studyEntryStudyEntryData", new EditLinkCell(i18n.get("StudyEntryDataView.heading"), studyEntryDataClickListener));
		addGeneratedColumn("remove", new RemoveCell<>(StudyEntryService.class));

		addContainerProperty("studyEntryType", StudyEntryType.class, null);
		addContainerProperty("time", DateTime.class, null);
		addContainerProperty("description", String.class, "");
		addContainerProperty("isRemoveCellEnabled", Boolean.class, false);
		addContainerProperty("isEditCellEnabled", Boolean.class, false);

		baseTable.setColumnHeader("studyEntryType", i18n.get("StudyEntry.studyEntryType"));
		baseTable.setColumnHeader("time", i18n.get("StudyEntry.time"));
		baseTable.setColumnHeader("studyEntryStudyEntryData", i18n.get("StudyEntryDataView.heading"));
		baseTable.setColumnHeader("description", i18n.get("StudyEntry.description"));
		baseTable.setColumnHeader("remove", "");
	}

	@Override
	protected StudyEntry getFormEntity() {
		StudyEntry studyEntry = new StudyEntry();
		studyEntry.setPatient(patient);
		return studyEntry;
	}

	@Override
	protected void generateForm(StudyEntry tEntity) {
		if (tEntity.getId() != null) {
			tEntity = studyEntryService.findOneWithRelationTable(tEntity.getId());
		}
		form = new StudyEntryForm(tEntity, true);
	}

	@Override
	protected void setGridDataSource() {
		setPatient();

		StudyEntryFilter sessionStudyEntryFilter = VaadinSession.getCurrent().getAttribute(StudyEntryFilter.class);
		if (sessionStudyEntryFilter != null) {
			studyEntryFilter = sessionStudyEntryFilter;
			VaadinSession.getCurrent().setAttribute(StudyEntryFilter.class, null);
		}

		if (FilterUtil.isFilterActive(studyEntryFilter)) {
			try {
				filterInfoVerticalLayout.removeComponent(filterInfo);
			} catch (Exception e) {
			}
			filterInfo = new FilterInfo<>(studyEntryFilter);
			filterInfoVerticalLayout.addComponent(filterInfo, 0);
		} else {
			try {
				quickSearchTextField.setValue("");
				filterInfoVerticalLayout.removeComponent(filterInfo);
			} catch (Exception e) {
			}
		}

		studyEntryFilter.setPatient(patient);

		List<StudyEntry> studyEntries = studyEntryService.findAllWithFilter(studyEntryFilter);
		if (filterInfo != null) {
			filterInfo.setEntityCount(studyEntries.size());
		}

		baseTable.setContainerDataSource(null);
		for (StudyEntry studyEntry : studyEntries) {
			Item item = indexedContainer.addItem(studyEntry.getId());
			item.getItemProperty("studyEntryType").setValue(studyEntry.getStudyEntryType());
			item.getItemProperty("time").setValue(studyEntry.getTime());
			item.getItemProperty("description").setValue(studyEntry.getDescription());
			if (studyEntryService.userHasPermission(PermissionName.PERM_STUDY_ENTRY_DELETE_ANY)) {
				item.getItemProperty("isRemoveCellEnabled").setValue(Boolean.TRUE);
			}
			if (studyEntryService.userHasPermission(PermissionName.PERM_STUDY_ENTRY_UPDATE_ANY)) {
				item.getItemProperty("isEditCellEnabled").setValue(Boolean.TRUE);
			}
		}
		baseTable.setContainerDataSource(indexedContainer);
		setVisibleColumns(visibleColumns.toArray());
	}

	@Override
	protected void setBaseTable() {
		baseTable = new BaseTable(null, indexedContainer);
		baseTable.setCacheRate(2);

		if (filterInfoVerticalLayout.getComponentCount() != 0) {
			filterInfoVerticalLayout = new VerticalLayout();
			studyEntryFilter = new StudyEntryFilter();
		}

		filterInfoVerticalLayout.setImmediate(true);
		filterInfoVerticalLayout.setSizeFull();
		filterInfoVerticalLayout.addComponent(baseTable);
		filterInfoVerticalLayout.setExpandRatio(baseTable, 1);
		addComponent(filterInfoVerticalLayout);
		setExpandRatio(filterInfoVerticalLayout, 1);
	}

	@Override
	public void windowClose(Window.CloseEvent closeEvent) {

	}

	private void setPatient() {
		if (UserUtil.getUser() instanceof Patient) {
			patient = (Patient) UserUtil.getUser();
		} else {
			if (VaadinSession.getCurrent().getAttribute(Patient.class) != null) {
				patient = VaadinSession.getCurrent().getAttribute(Patient.class);
			}
		}

		VaadinSession.getCurrent().setAttribute(Patient.class, null);

		if (!(UserUtil.getUser() instanceof Patient)) {
			String patientId = (patient != null && patient.getPatientId() != null) ? patient.getPatientId() : "";
			h1.setValue(i18n.get("StudyEntriesView.heading") + " (" + patientId + ")");
		}
	}

	private Component getQuickSearchComponent() {
		quickSearchTextField.addStyleName(ValoTheme.TEXTFIELD_SMALL);
		quickSearchTextField.setInputPrompt(i18n.get("BaseForm.quickSearch"));
		quickSearchTextField.addValueChangeListener(quickSearchValueChangeListener);

		return quickSearchTextField;
	}

	private ComboBox getFilterComboBox() {
		filterComboBox.setWidth(90, Unit.PIXELS);
		filterComboBox.addStyleName(ValoTheme.COMBOBOX_SMALL);
		filterComboBox.setNullSelectionAllowed(false);
		filterComboBox.setTextInputAllowed(false);
		filterComboBox.setInputPrompt(i18n.get("BaseView.filterLabel"));
		setFilterComboBoxItems();
		filterComboBox.addValueChangeListener(filterComboBoxValueChangeListener);

		return filterComboBox;
	}

	private void setFilterComboBoxItems() {
		filterComboBox.removeAllItems();
		for (StudyEntryFilter studyEntryFilter : filterService.findAllByUser(UserUtil.getUser(), FilterCategory.STUDY_ENTRY, StudyEntryFilter.class)) {
			filterComboBox.addItem(studyEntryFilter);
			filterComboBox.setItemCaption(studyEntryFilter, studyEntryFilter.getFilterName());
		}

		filterComboBox.addItem("showFilter");
		filterComboBox.setItemCaption("showFilter", i18n.get("BaseView.showFilterLabel"));
	}

	private ComboBox getOtherComboBox() {
		otherComboBox.setWidth(90, Unit.PIXELS);
		otherComboBox.addStyleName(ValoTheme.COMBOBOX_SMALL);
		otherComboBox.setNullSelectionAllowed(false);
		otherComboBox.setTextInputAllowed(false);
		otherComboBox.setInputPrompt(i18n.get("GridView.otherLabel"));

		otherComboBox.addItem(BulkAction.EXPORT);
		otherComboBox.setItemCaption(BulkAction.EXPORT, i18n.get("GridView.otherExportLabel"));

		otherComboBox.addValueChangeListener(otherComboBoxValueChangeListener);

		return otherComboBox;
	}

	@EventBusListenerMethod
	public void buttonClick(EventBusAction eventBusAction) {
		if (eventBusAction.getFilter() != null && StudyEntryFilter.class.getName().equals(eventBusAction.getFilter().getClass().getName())) {
			if (eventBusAction.getAction().equals(Action.FILTER)) {
				studyEntryFilter = (StudyEntryFilter) eventBusAction.getFilter();
				refreshTable();
			} else if (eventBusAction.getAction().equals(Action.DELETE_FILTER)) {
				quickSearchTextField.removeValueChangeListener(quickSearchValueChangeListener);
				studyEntryFilter = (StudyEntryFilter) eventBusAction.getFilter();
				quickSearchTextField.setValue("");
				quickSearchTextField.addValueChangeListener(quickSearchValueChangeListener);
				refreshTable();
			} else if (eventBusAction.getAction().equals(Action.SAVE_FILTER)) {
				setFilterComboBoxItems();
			}
		}
	}
}
