package hu.oroscom.c1801.view;

import com.vaadin.data.Item;
import com.vaadin.data.Property;
import com.vaadin.server.FontAwesome;
import com.vaadin.server.VaadinSession;
import com.vaadin.ui.*;
import com.vaadin.ui.themes.ValoTheme;
import hu.oroscom.c1801.NeededAnyPermission;
import hu.oroscom.c1801.component.FilterInfo;
import hu.oroscom.c1801.component.cell.EditCell;
import hu.oroscom.c1801.component.cell.MultiSelectCell;
import hu.oroscom.c1801.component.cell.RemoveCell;
import hu.oroscom.c1801.component.form.AssignmentPanel;
import hu.oroscom.c1801.component.form.StudyEntryDataForm;
import hu.oroscom.c1801.component.table.BaseTable;
import hu.oroscom.c1801.component.windowed.FilterPanel;
import hu.oroscom.c1801.domain.StudyEntryData;
import hu.oroscom.c1801.domain.filter.StudyEntryDataFilter;
import hu.oroscom.c1801.enumeration.Action;
import hu.oroscom.c1801.enumeration.BulkAction;
import hu.oroscom.c1801.enumeration.FilterCategory;
import hu.oroscom.c1801.enumeration.PermissionName;
import hu.oroscom.c1801.service.*;
import hu.oroscom.c1801.util.EventBusAction;
import hu.oroscom.c1801.util.FilterUtil;
import hu.oroscom.c1801.util.LoggingExcelExport;
import hu.oroscom.c1801.util.UserUtil;
import org.vaadin.spring.annotation.VaadinUIScope;
import org.vaadin.spring.events.annotation.EventBusListenerMethod;
import org.vaadin.spring.navigator.annotation.VaadinView;

import javax.inject.Inject;
import java.util.*;

@VaadinUIScope
@VaadinView(name = StudyEntryDataView.NAME)
@NeededAnyPermission(values = {
		PermissionName.PERM_STUDY_ENTRY_DATA_SELECT_ANY
})
public class StudyEntryDataView extends GridView<StudyEntryData, StudyEntryDataService, StudyEntryDataForm> {
	public static final String NAME = "studyEntryData";
	private final List<Object> visibleColumns = new ArrayList<>(Arrays.asList(new Object[]{"multiSelect", "name", "dataKey", "description", "remove"}));

	private Button selectAllButton = new Button();
	private ComboBox bulkActionComboBox = new ComboBox();
	private ComboBox otherComboBox = new ComboBox();
	private ComboBox filterComboBox = new ComboBox();
	private TextField quickSearchTextField = new TextField();
	private StudyEntryDataFilter studyEntryDataFilter = new StudyEntryDataFilter();
	private VerticalLayout filterInfoVerticalLayout = new VerticalLayout();
	private FilterInfo filterInfo;

	@Inject
	private StudyEntryDataService studyEntryDataService;

	@Inject
	private StudyEntryBaseDataService studyEntryBaseDataService;

	@Inject
	private StudyEntryService studyEntryService;

	@Inject
	private FilterService filterService;

	@Inject
	private ExcelExportLogService excelExportLogService;

	private final Property.ValueChangeListener filterComboBoxValueChangeListener = new Property.ValueChangeListener() {
		@Override
		public void valueChange(Property.ValueChangeEvent valueChangeEvent) {
			if ("showFilter".equals(valueChangeEvent.getProperty().getValue())) {
				FilterPanel filterPanel = new FilterPanel(studyEntryDataFilter, filterService, studyEntryDataService);
				addContentToWindow(filterPanel);
			} else {
				studyEntryDataFilter = (StudyEntryDataFilter) valueChangeEvent.getProperty().getValue();
				refreshTable();
			}

			filterComboBox.removeValueChangeListener(filterComboBoxValueChangeListener);
			filterComboBox.setValue(null);
			filterComboBox.addValueChangeListener(filterComboBoxValueChangeListener);
		}
	};

	private final Property.ValueChangeListener quickSearchValueChangeListener = new Property.ValueChangeListener() {
		@Override
		public void valueChange(Property.ValueChangeEvent event) {
			String searchText = event.getProperty().getValue().toString();
			if (searchText.length() > 2) {
				studyEntryDataFilter.setQuickSearch(searchText);
				eventBus.publish(this, new EventBusAction<>(Action.FILTER, studyEntryDataFilter));
			} else {
				Notification.show(i18n.get("QuickSearch.minThreeCharacter"), Notification.Type.WARNING_MESSAGE);
			}
		}
	};

	private final Property.ValueChangeListener otherComboBoxValueChangeListener = new Property.ValueChangeListener() {
		@Override
		public void valueChange(Property.ValueChangeEvent valueChangeEvent) {
			final BulkAction bulkAction = (BulkAction) valueChangeEvent.getProperty().getValue();
			switch (bulkAction) {
				case EXPORT:
					LoggingExcelExport excelExport = new LoggingExcelExport(baseTable, i18n.get("StudyEntryDataView.heading"), i18n.get("StudyEntryDataView.heading"), "studyEntryData.xls", false, excelExportLogService, i18n.get("StudyEntryDataView.heading"));
					excelExport.excludeCollapsedColumns();
					Iterator iterator = excelExport.getPropIds().iterator();

					while (iterator.hasNext()) {
						Object propId = iterator.next();
						if ("remove".equals(propId)) {
							iterator.remove();
						}
					}
					excelExport.export();
					break;
			}

			otherComboBox.removeValueChangeListener(otherComboBoxValueChangeListener);
			otherComboBox.setValue(null);
			otherComboBox.addValueChangeListener(otherComboBoxValueChangeListener);
		}
	};

	private Button.ClickListener selectAllButtonClickListener = new Button.ClickListener() {
		@Override
		public void buttonClick(Button.ClickEvent event) {
			boolean isSelectedItemIdsEmpty = getSelectedItemIds().isEmpty();
			if (isSelectedItemIdsEmpty) {
				getSelectedItemIds().addAll(indexedContainer.getItemIds());
			} else {
				getSelectedItemIds().clear();
			}
			for (Object itemId : indexedContainer.getItemIds()) {
				indexedContainer.getItem(itemId).getItemProperty("multiSelect").setValue(isSelectedItemIdsEmpty);
			}
			baseTable.refreshRowCache();
		}
	};

	private final Property.ValueChangeListener bulkActionComboBoxValueChangeListener = new Property.ValueChangeListener() {
		@Override
		public void valueChange(Property.ValueChangeEvent valueChangeEvent) {
			if (getSelectedItemIds().isEmpty() || valueChangeEvent.getProperty().getValue() == null) {
				bulkActionComboBox.setValue(null);
				Notification.show(i18n.get("GridView.noSelectedEntity"), Notification.Type.HUMANIZED_MESSAGE);
				return;
			}

			final Set<Object> selectedItemIds = getSelectedItemIds();

			if (!selectedItemIds.isEmpty()) {
				Component component = null;
				final BulkAction bulkAction = (BulkAction) valueChangeEvent.getProperty().getValue();
				switch (bulkAction) {
					case STUDY_ENTRY_TYPE:
						component = generateAssignmentPanel();
						break;
				}
				addContentToWindow(component);
			}

			bulkActionComboBox.removeValueChangeListener(bulkActionComboBoxValueChangeListener);
			bulkActionComboBox.setValue(null);
			bulkActionComboBox.addValueChangeListener(bulkActionComboBoxValueChangeListener);
		}
	};

	private Component generateAssignmentPanel() {
		return new AssignmentPanel(getSelectedItemIds(), studyEntryService, studyEntryDataService, studyEntryBaseDataService);
	}

	@Override
	protected void init() {
		super.defaultComponents(i18n.get("StudyEntryDataView.heading"));
		initTable();
	}

	@Override
	protected void customizeView() {
		Label expandingGap = new Label();
		expandingGap.setWidth("100%");

		getHeaderToolBarHorizontalLayout().setWidth(100, Unit.PERCENTAGE);
		getHeaderToolBarHorizontalLayout().setSpacing(true);
		getHeaderToolBarHorizontalLayout().addComponent(getSelectAllButton());
		getHeaderToolBarHorizontalLayout().addComponent(getBulkActionComboBox());
		getHeaderToolBarHorizontalLayout().addComponent(getQuickSearchComponent());
		getHeaderToolBarHorizontalLayout().addComponent(getFilterComboBox());
		getHeaderToolBarHorizontalLayout().addComponent(expandingGap);
		getHeaderToolBarHorizontalLayout().addComponent(getOtherComboBox());
		getHeaderToolBarHorizontalLayout().setExpandRatio(expandingGap, 1);

		if (studyEntryDataService.userHasPermission(PermissionName.PERM_STUDY_ENTRY_DATA_CREATE_ANY)) {
			setAddButton(i18n.get("StudyEntryDataView.addButton"));
		}
	}

	@Override
	protected void registerColumns() {
		baseTable.addStyleName("studyEntryDataView-view");
		baseTable.setHeight(100, Unit.PERCENTAGE);
		baseTable.addStyleName(ValoTheme.TABLE_SMALL);
		baseTable.setColumnCollapsingAllowed(true);
		baseTable.setColumnReorderingAllowed(true);

		addGeneratedColumn("multiSelect", new MultiSelectCell(getSelectedItemIds()));
		addGeneratedColumn("name", new EditCell(editClickListener));
		addGeneratedColumn("remove", new RemoveCell<>(StudyEntryService.class));

		addContainerProperty("multiSelect", Boolean.class, false);
		addContainerProperty("name", String.class, "");
		addContainerProperty("dataKey", String.class, "");
		addContainerProperty("description", String.class, "");
		addContainerProperty("isRemoveCellEnabled", Boolean.class, false);
		addContainerProperty("isEditCellEnabled", Boolean.class, false);

		baseTable.setColumnHeader("multiSelect", "");
		baseTable.setColumnHeader("remove", "");
		baseTable.setColumnHeader("name", i18n.get("StudyEntryData.name"));
		baseTable.setColumnHeader("dataKey", i18n.get("StudyEntryData.dataKey"));
		baseTable.setColumnHeader("description", i18n.get("StudyEntryData.description"));

		baseTable.setColumnWidth("multiSelect", 28);
	}

	@Override
	protected StudyEntryData getFormEntity() {
		return new StudyEntryData();
	}

	@Override
	protected void generateForm(StudyEntryData tEntity) {
		form = new StudyEntryDataForm(tEntity, true);
	}

	@Override
	protected void setGridDataSource() {
		StudyEntryDataFilter sessionStudyEntryDataFilter = VaadinSession.getCurrent().getAttribute(StudyEntryDataFilter.class);
		if (sessionStudyEntryDataFilter != null) {
			studyEntryDataFilter = sessionStudyEntryDataFilter;
			VaadinSession.getCurrent().setAttribute(StudyEntryDataFilter.class, null);
		}

		if (FilterUtil.isFilterActive(studyEntryDataFilter)) {
			try {
				filterInfoVerticalLayout.removeComponent(filterInfo);
			} catch (Exception e) {
			}
			filterInfo = new FilterInfo<>(studyEntryDataFilter);
			filterInfoVerticalLayout.addComponent(filterInfo, 0);
		} else {
			try {
				quickSearchTextField.setValue("");
				filterInfoVerticalLayout.removeComponent(filterInfo);
			} catch (Exception e) {
			}
		}

		List<StudyEntryData> studyEntryDataList = studyEntryDataService.findAllWithFilter(studyEntryDataFilter);
		if (filterInfo != null) {
			filterInfo.setEntityCount(studyEntryDataList.size());
		}

		baseTable.setContainerDataSource(null);
		for (StudyEntryData studyEntryData : studyEntryDataList) {
			Item item = indexedContainer.addItem(studyEntryData.getId());
			item.getItemProperty("multiSelect").setValue(Boolean.FALSE);
			item.getItemProperty("name").setValue(studyEntryData.getName());
			item.getItemProperty("dataKey").setValue(studyEntryData.getDataKey());
			item.getItemProperty("description").setValue(studyEntryData.getDescription());
			if (studyEntryDataService.userHasPermission(PermissionName.PERM_STUDY_ENTRY_DATA_DELETE_ANY)) {
				item.getItemProperty("isRemoveCellEnabled").setValue(Boolean.TRUE);
			}
			if (studyEntryDataService.userHasPermission(PermissionName.PERM_STUDY_ENTRY_DATA_UPDATE_ANY)) {
				item.getItemProperty("isEditCellEnabled").setValue(Boolean.TRUE);
			}
		}
		baseTable.setContainerDataSource(indexedContainer);
		setVisibleColumns(visibleColumns.toArray());
	}

	@Override
	protected void setBaseTable() {
		baseTable = new BaseTable(null, indexedContainer);
		baseTable.setCacheRate(2);

		if (filterInfoVerticalLayout.getComponentCount() != 0) {
			filterInfoVerticalLayout = new VerticalLayout();
			studyEntryDataFilter = new StudyEntryDataFilter();
		}

		filterInfoVerticalLayout.setImmediate(true);
		filterInfoVerticalLayout.setSizeFull();
		filterInfoVerticalLayout.addComponent(baseTable);
		filterInfoVerticalLayout.setExpandRatio(baseTable, 1);
		addComponent(filterInfoVerticalLayout);
		setExpandRatio(filterInfoVerticalLayout, 1);
	}

	@Override
	public void windowClose(Window.CloseEvent closeEvent) {

	}

	private Button getSelectAllButton() {
		selectAllButton.setIcon(FontAwesome.CHECK_SQUARE);
		selectAllButton.addStyleName(ValoTheme.BUTTON_SMALL);
		selectAllButton.addStyleName(ValoTheme.BUTTON_ICON_ONLY);
		selectAllButton.addClickListener(selectAllButtonClickListener);

		return selectAllButton;
	}

	private ComboBox getBulkActionComboBox() {
		bulkActionComboBox.setWidth(160, Unit.PIXELS);
		bulkActionComboBox.addStyleName(ValoTheme.COMBOBOX_SMALL);
		bulkActionComboBox.setNullSelectionAllowed(false);
		bulkActionComboBox.setTextInputAllowed(false);
		bulkActionComboBox.setInputPrompt(i18n.get("GridView.bulkActionLabel"));

		bulkActionComboBox.addItem(BulkAction.STUDY_ENTRY_TYPE);
		bulkActionComboBox.setItemCaption(BulkAction.STUDY_ENTRY_TYPE, BulkAction.STUDY_ENTRY_TYPE.getCaption());

		bulkActionComboBox.addValueChangeListener(bulkActionComboBoxValueChangeListener);

		return bulkActionComboBox;
	}

	private Component getQuickSearchComponent() {
		quickSearchTextField.addStyleName(ValoTheme.TEXTFIELD_SMALL);
		quickSearchTextField.setInputPrompt(i18n.get("BaseForm.quickSearch"));
		quickSearchTextField.addValueChangeListener(quickSearchValueChangeListener);

		return quickSearchTextField;
	}

	private ComboBox getFilterComboBox() {
		filterComboBox.setWidth(90, Unit.PIXELS);
		filterComboBox.addStyleName(ValoTheme.COMBOBOX_SMALL);
		filterComboBox.setNullSelectionAllowed(false);
		filterComboBox.setTextInputAllowed(false);
		filterComboBox.setInputPrompt(i18n.get("BaseView.filterLabel"));
		setFilterComboBoxItems();
		filterComboBox.addValueChangeListener(filterComboBoxValueChangeListener);

		return filterComboBox;
	}

	private void setFilterComboBoxItems() {
		filterComboBox.removeAllItems();
		for (StudyEntryDataFilter studyEntryDataFilter : filterService.findAllByUser(UserUtil.getUser(), FilterCategory.STUDY_ENTRY_DATA, StudyEntryDataFilter.class)) {
			filterComboBox.addItem(studyEntryDataFilter);
			filterComboBox.setItemCaption(studyEntryDataFilter, studyEntryDataFilter.getFilterName());
		}

		filterComboBox.addItem("showFilter");
		filterComboBox.setItemCaption("showFilter", i18n.get("BaseView.showFilterLabel"));
	}

	private ComboBox getOtherComboBox() {
		otherComboBox.setWidth(90, Unit.PIXELS);
		otherComboBox.addStyleName(ValoTheme.COMBOBOX_SMALL);
		otherComboBox.setNullSelectionAllowed(false);
		otherComboBox.setTextInputAllowed(false);
		otherComboBox.setInputPrompt(i18n.get("GridView.otherLabel"));

		otherComboBox.addItem(BulkAction.EXPORT);
		otherComboBox.setItemCaption(BulkAction.EXPORT, i18n.get("GridView.otherExportLabel"));

		otherComboBox.addValueChangeListener(otherComboBoxValueChangeListener);

		return otherComboBox;
	}

	@EventBusListenerMethod
	public void buttonClick(EventBusAction eventBusAction) {
		if (eventBusAction.getFilter() != null && StudyEntryDataFilter.class.getName().equals(eventBusAction.getFilter().getClass().getName())) {
			if (eventBusAction.getAction().equals(Action.FILTER)) {
				studyEntryDataFilter = (StudyEntryDataFilter) eventBusAction.getFilter();
				refreshTable();
			} else if (eventBusAction.getAction().equals(Action.DELETE_FILTER)) {
				quickSearchTextField.removeValueChangeListener(quickSearchValueChangeListener);
				studyEntryDataFilter = (StudyEntryDataFilter) eventBusAction.getFilter();
				quickSearchTextField.setValue("");
				quickSearchTextField.addValueChangeListener(quickSearchValueChangeListener);
				refreshTable();
			} else if (eventBusAction.getAction().equals(Action.SAVE_FILTER)) {
				setFilterComboBoxItems();
			}
		}
	}
}
