package hu.oroscom.c1801.view;

import com.vaadin.data.Item;
import com.vaadin.data.Property;
import com.vaadin.server.FontAwesome;
import com.vaadin.server.VaadinSession;
import com.vaadin.ui.*;
import com.vaadin.ui.themes.ValoTheme;
import hu.oroscom.c1801.NeededAnyPermission;
import hu.oroscom.c1801.component.FilterInfo;
import hu.oroscom.c1801.component.cell.ButtonCell;
import hu.oroscom.c1801.component.cell.DateTimeFormatCell;
import hu.oroscom.c1801.component.cell.EditCell;
import hu.oroscom.c1801.component.cell.RemoveCell;
import hu.oroscom.c1801.component.form.UserCommentForm;
import hu.oroscom.c1801.component.table.BaseTable;
import hu.oroscom.c1801.component.windowed.FilterPanel;
import hu.oroscom.c1801.domain.Patient;
import hu.oroscom.c1801.domain.UserComment;
import hu.oroscom.c1801.domain.filter.StudyEntryDataFilter;
import hu.oroscom.c1801.domain.filter.UserCommentFilter;
import hu.oroscom.c1801.enumeration.*;
import hu.oroscom.c1801.service.ExcelExportLogService;
import hu.oroscom.c1801.service.FilterService;
import hu.oroscom.c1801.service.StudyEntryService;
import hu.oroscom.c1801.service.UserCommentService;
import hu.oroscom.c1801.util.EventBusAction;
import hu.oroscom.c1801.util.FilterUtil;
import hu.oroscom.c1801.util.LoggingExcelExport;
import hu.oroscom.c1801.util.UserUtil;
import org.joda.time.DateTime;
import org.vaadin.spring.annotation.VaadinUIScope;
import org.vaadin.spring.events.annotation.EventBusListenerMethod;
import org.vaadin.spring.navigator.annotation.VaadinView;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

@VaadinUIScope
@VaadinView(name = UserCommentsView.NAME)
@NeededAnyPermission(values = {
		PermissionName.PERM_USER_COMMENT_SELECT_ANY
})
public class UserCommentsView extends GridView<UserComment, UserCommentService, UserCommentForm> {
	public static final String NAME = "userComment";
	private final List<Object> visibleColumns = new ArrayList<>(Arrays.asList(new Object[]{"forward", "question", "answer", "insertedAt", "remove"}));

//	private Button selectAllButton = new Button();
	private ComboBox otherComboBox = new ComboBox();
	private ComboBox filterComboBox = new ComboBox();
	private TextField quickSearchTextField = new TextField();
	private UserCommentFilter userCommentFilter = new UserCommentFilter();
	private VerticalLayout filterInfoVerticalLayout = new VerticalLayout();
	private FilterInfo filterInfo;

	@Inject
	private UserCommentService userCommentService;

	@Inject
	private FilterService filterService;

	@Inject
	private ExcelExportLogService excelExportLogService;

	private final Property.ValueChangeListener filterComboBoxValueChangeListener = new Property.ValueChangeListener() {
		@Override
		public void valueChange(Property.ValueChangeEvent valueChangeEvent) {
			if ("showFilter".equals(valueChangeEvent.getProperty().getValue())) {
				FilterPanel filterPanel = new FilterPanel(userCommentFilter, filterService, userCommentService);
				addContentToWindow(filterPanel);
			} else {
				userCommentFilter = (UserCommentFilter) valueChangeEvent.getProperty().getValue();
				refreshTable();
			}

			filterComboBox.removeValueChangeListener(filterComboBoxValueChangeListener);
			filterComboBox.setValue(null);
			filterComboBox.addValueChangeListener(filterComboBoxValueChangeListener);
		}
	};

	private final Property.ValueChangeListener quickSearchValueChangeListener = new Property.ValueChangeListener() {
		@Override
		public void valueChange(Property.ValueChangeEvent event) {
			String searchText = event.getProperty().getValue().toString();
			if (searchText.length() > 2) {
				userCommentFilter.setQuickSearch(searchText);
				eventBus.publish(this, new EventBusAction<>(Action.FILTER, userCommentFilter));
			} else {
				Notification.show(i18n.get("QuickSearch.minThreeCharacter"), Notification.Type.WARNING_MESSAGE);
			}
		}
	};

	private final Property.ValueChangeListener otherComboBoxValueChangeListener = new Property.ValueChangeListener() {
		@Override
		public void valueChange(Property.ValueChangeEvent valueChangeEvent) {
			final BulkAction bulkAction = (BulkAction) valueChangeEvent.getProperty().getValue();
			switch (bulkAction) {
				case EXPORT:
					LoggingExcelExport excelExport = new LoggingExcelExport(baseTable, i18n.get("UserCommentsView.heading"), i18n.get("UserCommentsView.heading"), "userCommentsView.xls", false, excelExportLogService, i18n.get("UserCommentsView.heading"));
					excelExport.excludeCollapsedColumns();
					Iterator iterator = excelExport.getPropIds().iterator();

					while (iterator.hasNext()) {
						Object propId = iterator.next();
						if ("remove".equals(propId)) {
							iterator.remove();
						}
					}
					excelExport.export();
					break;
			}

			otherComboBox.removeValueChangeListener(otherComboBoxValueChangeListener);
			otherComboBox.setValue(null);
			otherComboBox.addValueChangeListener(otherComboBoxValueChangeListener);
		}
	};

	private final Button.ClickListener forwardClickListener = new Button.ClickListener() {
		@Override
		public void buttonClick(Button.ClickEvent event) {
			Long userCommentId = (Long) event.getButton().getData();
			UserComment userComment = userCommentService.findOne(userCommentId);
			userComment.setUserCommentType(UserCommentType.ASSIGNED_TO_DOCTOR);
			userCommentService.save(userComment);
			refreshTable();
		}
	};

	private final Button.ClickListener viewClickListener = new Button.ClickListener() {
		@Override
		public void buttonClick(Button.ClickEvent event) {
			Long userCommentId = (Long) event.getButton().getData();
			UserComment userComment = userCommentService.findOne(userCommentId);
			userComment.setViewed(true);
			userCommentService.save(userComment);
			refreshTable();
		}
	};

	@Override
	protected void init() {
		super.defaultComponents(i18n.get("UserCommentsView.heading"));
		initTable();
	}

	@Override
	protected void customizeView() {
		Label expandingGap = new Label();
		expandingGap.setWidth("100%");

		getHeaderToolBarHorizontalLayout().setWidth(100, Unit.PERCENTAGE);
		getHeaderToolBarHorizontalLayout().setSpacing(true);
		getHeaderToolBarHorizontalLayout().addComponent(getQuickSearchComponent());
		getHeaderToolBarHorizontalLayout().addComponent(getFilterComboBox());
		getHeaderToolBarHorizontalLayout().addComponent(expandingGap);
		getHeaderToolBarHorizontalLayout().addComponent(getOtherComboBox());
		getHeaderToolBarHorizontalLayout().setExpandRatio(expandingGap, 1);

		if (userCommentService.userHasPermission(PermissionName.PERM_USER_COMMENT_CREATE_ANY)) {
			setAddButton(i18n.get("UserCommentsView.addButton"));
		}
	}

	@Override
	protected void registerColumns() {
		baseTable.addStyleName("userCommentsView-view");
		baseTable.setHeight(100, Unit.PERCENTAGE);
		baseTable.addStyleName(ValoTheme.TABLE_SMALL);
		baseTable.setColumnCollapsingAllowed(true);
		baseTable.setColumnReorderingAllowed(true);

		addGeneratedColumn("question", new EditCell(editClickListener));
		addGeneratedColumn("insertedAt", new DateTimeFormatCell());
		if (UserUtil.isPatient()) {
			addGeneratedColumn("forward", new ButtonCell(FontAwesome.ENVELOPE, viewClickListener));
		} else {
			addGeneratedColumn("forward", new ButtonCell(FontAwesome.FORWARD, forwardClickListener));
		}
		addGeneratedColumn("remove", new RemoveCell<>(StudyEntryService.class));

		addContainerProperty("question", String.class, "");
		addContainerProperty("answer", String.class, "");
		addContainerProperty("insertedAt", DateTime.class, null);
		addContainerProperty("isRemoveCellEnabled", Boolean.class, false);
		addContainerProperty("isEditCellEnabled", Boolean.class, false);
		addContainerProperty("isButtonCellEnabled", Boolean.class, false);

		baseTable.setColumnHeader("remove", "");
		baseTable.setColumnHeader("forward", "");
		baseTable.setColumnHeader("question", i18n.get("UserComment.question"));
		baseTable.setColumnHeader("answer", i18n.get("UserComment.answer"));
		baseTable.setColumnHeader("insertedAt", i18n.get("UserComment.insertedAt"));

		baseTable.setColumnWidth("forward", 40);

		baseTable.setCellStyleGenerator(new Table.CellStyleGenerator() {
			@Override
			public String getStyle(Table table, Object itemId, Object propertyId) {
				UserComment userComment = userCommentService.findOne((Long) itemId);
				if (userComment.getUserCommentType() == null) {
					return null;
				}
				switch (userComment.getUserCommentType()) {
					case ASSIGNED_TO_DOCTOR:
						if (UserUtil.isAssistant()) {
							return "highlight-green";
						} else if (UserUtil.isDoctor()) {
							return "highlight-yellow";
						} else if (UserUtil.isPatient()) {
							return "highlight-yellow";
						}
						break;
					case ANSWERED:
						if (UserUtil.isPatient() && (UserCommentType.ANSWERED.equals(userComment.getUserCommentType()) || UserCommentType.BROADCAST.equals(userComment.getUserCommentType())) && !userComment.getViewed()) {
							return "highlight-green highlight-bold";
						}
						return "highlight-green";
					case BROADCAST:
						if (UserUtil.isAssistant()) {
							return "highlight-blue";
						} else if (UserUtil.isDoctor()) {
							return "highlight-green";
						} else if (UserUtil.isPatient() && (UserCommentType.ANSWERED.equals(userComment.getUserCommentType()) || UserCommentType.BROADCAST.equals(userComment.getUserCommentType())) && !userComment.getViewed()) {
							return "highlight-green highlight-bold";
						} else if (UserUtil.isPatient()) {
							return "highlight-green";
						}
						break;
				}
				return null;
			}
		});
	}

	@Override
	protected UserComment getFormEntity() {
		UserComment userComment = new UserComment();
		userComment.setCustomer(UserUtil.getCustomer());

		switch (UserUtil.getUser().getRole().getRoleNameEnum()) {
			case ROLE_PATIENT:
				Patient patient = (Patient) UserUtil.getUser();
				List<Patient> patients = new ArrayList<>();
				patients.add(patient);
				userComment.setPatients(patients);
				userComment.setDoctor(patient.getDoctor());
				break;
		}

		return userComment;
	}

	@Override
	protected void generateForm(UserComment tEntity) {
		form = new UserCommentForm(tEntity, true);
	}

	@Override
	protected void setGridDataSource() {
		UserCommentFilter sessionUserCommentFilter = VaadinSession.getCurrent().getAttribute(UserCommentFilter.class);
		if (sessionUserCommentFilter != null) {
			userCommentFilter = sessionUserCommentFilter;
			VaadinSession.getCurrent().setAttribute(UserCommentFilter.class, null);
		}

		if (FilterUtil.isFilterActive(userCommentFilter)) {
			try {
				filterInfoVerticalLayout.removeComponent(filterInfo);
			} catch (Exception e) {
			}
			filterInfo = new FilterInfo<>(userCommentFilter);
			filterInfoVerticalLayout.addComponent(filterInfo, 0);
		} else {
			try {
				quickSearchTextField.setValue("");
				filterInfoVerticalLayout.removeComponent(filterInfo);
			} catch (Exception e) {
			}
		}

		List<UserComment> userCommentList = userCommentService.findAllWithFilter(userCommentFilter);
		if (filterInfo != null) {
			filterInfo.setEntityCount(userCommentList.size());
		}

		baseTable.setContainerDataSource(null);
		for (UserComment userComment : userCommentList) {
			Item item = indexedContainer.addItem(userComment.getId());
			item.getItemProperty("question").setValue(userComment.getQuestion());
			item.getItemProperty("answer").setValue(userComment.getAnswer());
			item.getItemProperty("insertedAt").setValue(userComment.getInsertedAt());
			if (userCommentService.userHasPermission(PermissionName.PERM_USER_COMMENT_DELETE_ANY)) {
				item.getItemProperty("isRemoveCellEnabled").setValue(Boolean.TRUE);
			}
			if (userCommentService.userHasPermission(PermissionName.PERM_USER_COMMENT_UPDATE_ANY)) {
				if (!UserUtil.isPatient()) {
					item.getItemProperty("isEditCellEnabled").setValue(Boolean.TRUE);
				}
				if ((UserUtil.isAssistant() && userComment.getUserCommentType() == null) || (UserUtil.isPatient() && (UserCommentType.ANSWERED.equals(userComment.getUserCommentType()) || UserCommentType.BROADCAST.equals(userComment.getUserCommentType())) && !userComment.getViewed())) {
					item.getItemProperty("isButtonCellEnabled").setValue(Boolean.TRUE);
				}
			}
		}
		baseTable.setContainerDataSource(indexedContainer);
		setVisibleColumns(visibleColumns.toArray());
	}

	@Override
	protected void setBaseTable() {
		baseTable = new BaseTable(null, indexedContainer);
		baseTable.setCacheRate(2);

		if (filterInfoVerticalLayout.getComponentCount() != 0) {
			filterInfoVerticalLayout = new VerticalLayout();
			userCommentFilter = new UserCommentFilter();
		}

		filterInfoVerticalLayout.setImmediate(true);
		filterInfoVerticalLayout.setSizeFull();
		filterInfoVerticalLayout.addComponent(baseTable);
		filterInfoVerticalLayout.setExpandRatio(baseTable, 1);
		addComponent(filterInfoVerticalLayout);
		setExpandRatio(filterInfoVerticalLayout, 1);
	}

	@Override
	public void windowClose(Window.CloseEvent closeEvent) {

	}

	private Component getQuickSearchComponent() {
		quickSearchTextField.addStyleName(ValoTheme.TEXTFIELD_SMALL);
		quickSearchTextField.setInputPrompt(i18n.get("BaseForm.quickSearch"));
		quickSearchTextField.addValueChangeListener(quickSearchValueChangeListener);

		return quickSearchTextField;
	}

	private ComboBox getFilterComboBox() {
		filterComboBox.setWidth(90, Unit.PIXELS);
		filterComboBox.addStyleName(ValoTheme.COMBOBOX_SMALL);
		filterComboBox.setNullSelectionAllowed(false);
		filterComboBox.setTextInputAllowed(false);
		filterComboBox.setInputPrompt(i18n.get("BaseView.filterLabel"));
		setFilterComboBoxItems();
		filterComboBox.addValueChangeListener(filterComboBoxValueChangeListener);

		return filterComboBox;
	}

	private void setFilterComboBoxItems() {
		filterComboBox.removeAllItems();
		for (UserCommentFilter userCommentFilter : filterService.findAllByUser(UserUtil.getUser(), FilterCategory.USER_COMMENT, UserCommentFilter.class)) {
			filterComboBox.addItem(userCommentFilter);
			filterComboBox.setItemCaption(userCommentFilter, userCommentFilter.getFilterName());
		}

		filterComboBox.addItem("showFilter");
		filterComboBox.setItemCaption("showFilter", i18n.get("BaseView.showFilterLabel"));
	}

	private ComboBox getOtherComboBox() {
		otherComboBox.setWidth(90, Unit.PIXELS);
		otherComboBox.addStyleName(ValoTheme.COMBOBOX_SMALL);
		otherComboBox.setNullSelectionAllowed(false);
		otherComboBox.setTextInputAllowed(false);
		otherComboBox.setInputPrompt(i18n.get("GridView.otherLabel"));

		otherComboBox.addItem(BulkAction.EXPORT);
		otherComboBox.setItemCaption(BulkAction.EXPORT, i18n.get("GridView.otherExportLabel"));

		otherComboBox.addValueChangeListener(otherComboBoxValueChangeListener);

		return otherComboBox;
	}

	public void refreshView() {
		refreshTable();
	}

	@EventBusListenerMethod
	public void buttonClick(EventBusAction eventBusAction) {
		if (eventBusAction.getFilter() != null && StudyEntryDataFilter.class.getName().equals(eventBusAction.getFilter().getClass().getName())) {
			if (eventBusAction.getAction().equals(Action.FILTER)) {
				userCommentFilter = (UserCommentFilter) eventBusAction.getFilter();
				refreshTable();
			} else if (eventBusAction.getAction().equals(Action.DELETE_FILTER)) {
				quickSearchTextField.removeValueChangeListener(quickSearchValueChangeListener);
				userCommentFilter = (UserCommentFilter) eventBusAction.getFilter();
				quickSearchTextField.setValue("");
				quickSearchTextField.addValueChangeListener(quickSearchValueChangeListener);
				refreshTable();
			} else if (eventBusAction.getAction().equals(Action.SAVE_FILTER)) {
				setFilterComboBoxItems();
			}
		}
	}
}
